<?php
defined( 'ABSPATH' ) || exit;

class TMuserBlogBBP {
    /* The single instance of the class */
	protected static $_instance = null;

    /* Main Instance */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

    /* Constructor */
    public function __construct() {
        add_action( 'bbp_template_before_user_profile', array($this, 'before_content') );
        add_action( 'bbp_template_after_user_profile', array($this, 'after_content') );
        /* We used "wp_footer" because we need to check if the current user is the owner of the profile page */
        add_action( 'wp_footer', array($this, 'enqueue_scripts') );
    }

    /* Before Content */
    public function before_content() {
        if (isset($_GET["blog"]) && $_GET["blog"] == 1) {
            $form_id = tmuserb_get_option('tmuserb_options','bbpress_form_id', '');
            if (!empty($form_id)) {
                echo '<div id="tmuserb-bbpress" class="tmuserb-v-layout">';
                echo do_shortcode('[tmuserb_form form_id="' . $form_id .'"]');
                echo '</div>';
            } else {
                echo esc_html__( 'Please select form from TM User Blog settings.', 'tmuserblog' );
            }
        } else if(isset($_GET["blog"]) && $_GET["blog"] == 2) {
            $form_id = tmuserb_get_option('tmuserb_options','bbpress_form_id', '');
            if (!empty($form_id)) {
                $get_post_type = get_post_meta( $form_id, 'tmuserb_post_type', true );
                $user_id = bbp_get_displayed_user_id();
                echo '<div id="tmuserb-bbpress">';
                echo do_shortcode('[tmuserb_blog user_id="' . $user_id .'" post_type="' . $get_post_type .'"]');
                echo '</div>';
            } else {
                echo esc_html__( 'Please select form from TM User Blog settings.', 'tmuserblog' );
            }
        } else {
            return;
        }
    }

    /* After Content */
    public function after_content() {
        if (isset($_GET["blog"])) {
            echo '</div>';
        } else {
            return;
        }
    }

    /* Enqueue Scripts */
    public function enqueue_scripts() {
        $display = 'no';
        $page = '';
        $profile_url = bbp_get_user_profile_url();
        $user_id = get_current_user_id();
        $profile_id = bbp_get_displayed_user_id();

        if ($user_id === $profile_id) {
            $display = 'yes';
        }

        if (isset($_GET["blog"]) && $_GET["blog"] == 1) {
            $page = 'settings';
        } else if (isset($_GET["blog"]) && $_GET["blog"] == 2) {
            $page = 'blog';
        }

        wp_enqueue_script( 'tmuserb-bbpress', plugins_url( '/js/bbpress.js', __FILE__), array('jquery'), '1.0.3', true );
        $bbpress_param = array(
            "display" => $display,
            "page" => $page,
            "profileURL" => $profile_url,
            "menuTitle" => esc_html__( 'My Posts', 'tmuserblog' ),
            "menuTitle2" => esc_html__( 'Blog', 'tmuserblog' )
        );
        wp_localize_script('tmuserb-bbpress', 'tmuserb_bbpress_vars', $bbpress_param);
    }
}

/**
 * Returns the main instance of TMuserBlogBBP.
 */
function TMuserBlogBBP() {  
	return TMuserBlogBBP::instance();
}
// Global for backwards compatibility.
$GLOBALS['TMuserBlogBBP'] = TMuserBlogBBP();