<?php
defined( 'ABSPATH' ) || exit;

class TMuserBlogBP {
    /* The single instance of the class */
	protected static $_instance = null;

    /* Main Instance */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

    /* Constructor */
    public function __construct() {
        if (!empty($this->get_form_id())) {
            add_action( 'bp_setup_nav', array($this, 'user_tab'), 50 );
            add_action( 'bp_setup_admin_bar', array($this, 'admin_bar_menu'), 70 );
            add_action( 'publish_post', array($this, 'activity_add'),10 ,2 ); 
        }
    }

    /* Get Form ID */
    public function get_form_id() {
        $form_id = tmuserb_get_option('tmuserb_options','buddypress_form_id', '');
        return $form_id;
    }

    /* Add User Tab */
    public function user_tab() {
        bp_core_new_nav_item(
            array(
                'name'  =>  esc_html__( 'Blog', 'tmuserblog' ),
                'slug' => 'user-blog',
                'position' => 70,
                'screen_function' => array($this, 'get_author_template'),
                'default_subnav_slug' => 'user-blog',
                'show_for_displayed_user' => true
            )
        );
        bp_core_new_nav_item(
            array(
                'name'  =>  esc_html__( 'My Posts', 'tmuserblog' ),
                'slug' => 'my-posts',
                'position' => 71,
                'screen_function' => array($this, 'get_template'),
                'default_subnav_slug' => 'my-posts',
                'show_for_displayed_user' => false
            )
        );
    }

    /* Add Menu Item To The Admin Bar */      
    public function admin_bar_menu() {
        global $wp_admin_bar, $bp;
    
        if ( !bp_use_wp_admin_bar() || defined( 'DOING_AJAX' ) )
            return;
    
        $user_domain = bp_loggedin_user_domain();
        $item_link = trailingslashit( $user_domain . 'my-posts' );
    
        $wp_admin_bar->add_menu( array(
            'parent'  => $bp->my_account_menu_id,
            'id'      => 'my-posts',
            'title'   => esc_html__( 'My Posts', 'tmuserblog' ),
            'href'    => trailingslashit( $item_link )
        ));
    }

    /* Get The Templates  */
    public function get_template() {
        add_action( 'bp_template_content', array($this, 'get_content') );
        bp_core_load_template( apply_filters( 'bp_core_template_plugin', 'members/single/plugins' ) );
    }
    public function get_author_template() {
        add_action( 'bp_template_content', array($this, 'get_blog_content') );
        bp_core_load_template( apply_filters( 'bp_core_template_plugin', 'members/single/plugins' ) );
    }  
    
    /* Get The Content  */
    public function get_content() {
        if (!empty($this->get_form_id())) {
            echo '<div id="tmuserb-buddypress" class="tmuserb-v-layout">';
            echo do_shortcode('[tmuserb_form form_id="' . $this->get_form_id() .'"]');
            echo '</div>';
        } else {
            echo esc_html__( 'Please select form from TM User Blog settings.', 'tmuserblog' );
        }
    }
    public function get_blog_content() {
        if (!empty($this->get_form_id())) {
            $get_post_type = get_post_meta( $this->get_form_id(), 'tmuserb_post_type', true );
            $user_id = bp_displayed_user_id();
            echo '<div id="tmuserb-buddypress">'; 
            echo do_shortcode('[tmuserb_blog user_id="' . $user_id .'" post_type="' . $get_post_type .'"]');
            echo '</div>';
        } else {
            echo esc_html__( 'Please select form from TM User Blog settings.', 'tmuserblog' );
        }
    }
    
    /* Add Activity  */
    public function activity_add( $ID , $post ) {
        bp_activity_add(array(
            'action' => get_the_author_meta('display_name', $post->post_author) . ' ' . esc_html__( 'published', 'tmuserblog' ) . ' <a href="' . get_permalink($ID) . '">' . $post->post_title . '</a> ',
            'component' => 'activity',
            'type' => 'publish_post',
            'primary_link' => get_permalink($ID),
            'user_id' => $post->post_author,
            'content' => empty($post->post_excerpt) ? wp_trim_words($post->post_content, 55, '...') : $post->post_excerpt,
            'hide_sitewide' => false
        ));
    }

}

/**
 * Returns the main instance of TMuserBlogBP.
 */
function TMuserBlogBP() {  
	return TMuserBlogBP::instance();
}
// Global for backwards compatibility.
$GLOBALS['TMuserBlogBP'] = TMuserBlogBP();