<?php
defined( 'ABSPATH' ) || exit;

class TMuserBlogCPT {
    /* The single instance of the class */
	protected static $_instance = null;

    /* Main Instance */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

    /* Constructor */
    public function __construct() {
        add_action( 'init', array($this, 'post_type') );
        add_filter( 'cmb2_admin_init', array($this, 'register_meta_boxes') );
        add_action( 'admin_print_scripts-post-new.php', array($this, 'admin_scripts'), 11 );
        add_action( 'admin_print_scripts-post.php', array($this, 'admin_scripts'), 11 );
        add_filter( 'manage_edit-tmuserb_columns', array($this, 'add_admin_column'), 5);
        add_action( 'manage_posts_custom_column', array($this, 'add_admin_column_content'), 5, 2);
        add_action( 'admin_head', array($this, 'admin_styles'));
    }

    /* Admin scripts */

    public function admin_scripts() {
        global $post_type;
        if ( $post_type == 'tmuserb' ) {
            wp_enqueue_style('tmuserb-cpt', plugins_url( '/css/cpt.css', __FILE__), false, '1.0.1');
            wp_enqueue_script( 'tmuserb-cpt', plugins_url( '/js/cpt.js', __FILE__), array( 'jquery' ), '1.0', true ); 
        }
    }

    /* Register Custom Post Type */
    public function post_type() {
        $labels = array(
            'name'              => esc_html__( 'Post Forms', 'tmuserblog' ),
            'singular_name'     => esc_html__( 'Post Form', 'tmuserblog' ),
            'add_new'           => esc_html__( 'Add new form', 'tmuserblog' ),
            'add_new_item'      => esc_html__( 'Add new form', 'tmuserblog' ),
            'edit_item'         => esc_html__( 'Edit form', 'tmuserblog' ),
            'new_item'          => esc_html__( 'New form', 'tmuserblog' ),
            'view_item'         => esc_html__( 'View form', 'tmuserblog' ),
            'search_items'      => esc_html__( 'Search forms', 'tmuserblog' ),
            'not_found'         => esc_html__( 'No form found', 'tmuserblog' ),
            'not_found_in_trash'=> esc_html__( 'No form found in trash', 'tmuserblog' ),
            'parent_item_colon' => esc_html__( 'Parent form:', 'tmuserblog' ),
            'menu_name'         => esc_html__( 'Post Forms', 'tmuserblog' )
        );
    
        $taxonomies = array();
     
        $supports = array('title');
     
        $post_type_args = array(
            'labels'            => $labels,
            'singular_label'    => esc_html__('Post Form', 'tmuserblog'),
            'public'            => false,
            'exclude_from_search' => true,
            'show_ui'           => true,
            'show_in_nav_menus' => false,
            'show_in_rest' => false,
            'publicly_queryable'=> true,
            'query_var'         => true,
            'capability_type'   => 'post',
            'has_archive'       => false,
            'hierarchical'      => false,
            'rewrite'           => array( 'slug' => 'tmuserb', 'with_front' => false ),
            'supports'          => $supports,
            'capabilities' => array(
                'publish_posts' => 'manage_options',
                'edit_posts' => 'manage_options',
                'edit_others_posts' => 'manage_options',
                'delete_posts' => 'manage_options',
                'delete_others_posts' => 'manage_options',
                'read_private_posts' => 'manage_options',
                'edit_post' => 'manage_options',
                'delete_post' => 'manage_options',
                'read_post' => 'manage_options'
            ),
            'menu_position'     => 99,
            'menu_icon'         => 'dashicons-admin-post'
        );
        register_post_type('tmuserb',$post_type_args);
    }

    /* Register Metaboxes */
    public function register_meta_boxes( $meta_boxes ) {
        $prefix = 'tmuserb_cmb2'; // Prefix for all fields

        /* Form Fields Metabox */
        
        $fields_metabox = new_cmb2_box( 
            array(
                'id' => 'tmuserb_defaults',
                'title' => esc_html__( 'Form Fields', 'tmuserblog'),
                'object_types' => array('tmuserb'), // post type
                'context' => 'normal',
                'priority' => 'high',
                'show_names' => true, // Show field names on the left
            ) 
        );

        /* Default Fields */

        $fields_metabox->add_field(
            array(
                'name'    => esc_html__( 'Main Fields', 'tmuserblog'),
                'id'      => $prefix . '_defaults',
                'type' => 'multicheck_inline',
                'select_all_button' => false,
                'options' => array(
                    'post_format'   => esc_html__( 'Post Format', 'tmuserblog' ),
                    'content'   => esc_html__( 'Content', 'tmuserblog' ),
                    'excerpt'   => esc_html__( 'Excerpt', 'tmuserblog' ),
                    'featured'   => esc_html__( 'Featured Image', 'tmuserblog' )
                ),
                'default' => array('content', 'excerpt', 'featured')
            )
        );

        $fields_metabox->add_field(
            array(
                'name'    => esc_html__( 'Required Main Fields', 'tmuserblog'),
                'id'      => $prefix . '_req_main',
                'type' => 'multicheck_inline',
                'select_all_button' => false,
                'options' => array(
                    'post_format'   => esc_html__( 'Post Format', 'tmuserblog' ),
                    'content'   => esc_html__( 'Content', 'tmuserblog' ),
                    'excerpt'   => esc_html__( 'Excerpt', 'tmuserblog' ),
                    'featured'   => esc_html__( 'Featured Image', 'tmuserblog' )
                ),
                'default' => array('content')
            )
        );

        /* Additional Fields */

        $fields_metabox->add_field(
            array(
                'name'    => esc_html__( 'Additional Fields', 'tmuserblog'),
                'id'      => $prefix . '_additional_fields',
                'type' => 'radio_inline',
                'options' => array(
                    'enable' => esc_html__( 'Enable', 'tmuserblog' ),
                    'disable'   => esc_html__( 'Disable', 'tmuserblog' )
                ),
                'default' => 'disable',
            )
        );

        $additional_field_id = $fields_metabox->add_field( array(
            'id'          => $prefix . '_custom_fields',
            'type'        => 'group',
            'options'     => array(
                'group_title'       => esc_html__( 'Field {#}', 'tmuserblog' ),
                'add_button'        => esc_html__( 'Add Another Field', 'tmuserblog' ),
                'remove_button'     => esc_html__( 'Remove Field', 'tmuserblog' ),
                'sortable'          => true,
                'closed'         => true
            )
        ) );
        
        $fields_metabox->add_group_field( $additional_field_id, array(
            'name' => esc_html__( 'Label', 'tmuserblog' ),
            'id'   => 'label',
            'type' => 'text'
        ) );

        $fields_metabox->add_group_field( $additional_field_id, array(
            'name' => esc_html__( 'Description', 'tmuserblog' ),
            'id'   => 'desc',
            'type' => 'text'
        ) );

        $fields_metabox->add_group_field( $additional_field_id, array(
            'name' => esc_html__( 'Meta Key (Required)', 'tmuserblog' ),
            'desc' => esc_html__( 'Required for all field types except "Title".', 'tmuserblog' ),
            'id'   => 'meta_key',
            'type' => 'text'
        ) );

        $fields_metabox->add_group_field( $additional_field_id, array(
            'name' => esc_html__( 'Default Value', 'tmuserblog' ),
            'id'   => 'default_value',
            'type' => 'text'
        ) );

        $fields_metabox->add_group_field( $additional_field_id, array(
            'name' => esc_html__( 'Field Type', 'tmuserblog' ),
            'id'   => 'field_type',
            'type' => 'select',
            'options'  => array(
                'title' => esc_html__( 'Title', 'tmuserblog' ),
                'text' => esc_html__( 'Text', 'tmuserblog' ),
                'text_url' => esc_html__( 'URL', 'tmuserblog' ),
                'text_email' => esc_html__( 'Email', 'tmuserblog' ),
                'number' => esc_html__( 'Number', 'tmuserblog' ),
                'textarea' => esc_html__( 'Textarea', 'tmuserblog' ),
                'textarea_small' => esc_html__( 'Textarea (Small)', 'tmuserblog' ),
                'oembed' => esc_html__( 'Oembed', 'tmuserblog' ),
                'wysiwyg' => esc_html__( 'Rich Text Editor', 'tmuserblog' ),
                'colorpicker' => esc_html__( 'Colorpicker', 'tmuserblog' ),
                'select' => esc_html__( 'Select', 'tmuserblog' ),
                'radio' => esc_html__( 'Radio', 'tmuserblog' ),
            ),
            'default' => 'text'
        ) );

        $fields_metabox->add_group_field( $additional_field_id, array(
            'name' => esc_html__( 'Field Options', 'tmuserblog' ),
            'desc' => esc_html__( 'Required for Select and Radio field types. The data should be properly formatted like below;', 'tmuserblog' ) . '</br>' . esc_html__( 'key = value, key2 = value2, key3 = value3 ...', 'tmuserblog' ) . '</br>' . esc_html__( 'For example; red-light = Light Red, dark = Black, light = White', 'tmuserblog' ),
            'id'   => 'field_options',
            'type' => 'textarea_small'
        ) );

        $fields_metabox->add_group_field( $additional_field_id, array(
            'name' => esc_html__( 'Required Field', 'tmuserblog' ),
            'desc' => esc_html__( 'Make the field required.', 'tmuserblog' ),
            'id'   => 'required',
            'type' => 'radio_inline',
            'options'  => array(
                'enable' => esc_html__( 'Yes', 'tmuserblog' ),
                'disable' => esc_html__( 'No', 'tmuserblog' )
            ),
            'default' => 'disable'
        ) );

        /* Taxonomies Metabox */
        
        $tax_metabox = new_cmb2_box( 
            array(
                'id' => 'tmuserb_taxonomy',
                'title' => esc_html__( 'Taxonomies', 'tmuserblog'),
                'object_types' => array('tmuserb'), // post type
                'context' => 'normal',
                'priority' => 'default',
                'show_names' => true, // Show field names on the left
            ) 
        );

        /* Taxonomies */

        $tax_metabox->add_field(
            array(
                'name'    => esc_html__( 'Taxonomies', 'tmuserblog'),
                'id'      => $prefix . '_taxonomies',
                'type' => 'radio_inline',
                'options' => array(
                    'enable' => esc_html__( 'Enable', 'tmuserblog' ),
                    'disable'   => esc_html__( 'Disable', 'tmuserblog' )
                ),
                'default' => 'disable',
            )
        );

        $tax_field_id = $tax_metabox->add_field( array(
            'id'          => $prefix . '_tax',
            'type'        => 'group',
            'options'     => array(
                'group_title'       => esc_html__( 'Taxonomy {#}', 'tmuserblog' ),
                'add_button'        => esc_html__( 'Add Another Taxonomy', 'tmuserblog' ),
                'remove_button'     => esc_html__( 'Remove Taxonomy', 'tmuserblog' ),
                'sortable'          => true,
                'closed'         => true
            ),
        ) );
        
        $tax_metabox->add_group_field( $tax_field_id, array(
            'name' => esc_html__( 'Label', 'tmuserblog' ),
            'id'   => 'label',
            'type' => 'text'
        ) );

        $tax_metabox->add_group_field( $tax_field_id, array(
            'name' => esc_html__( 'Taxonomy', 'tmuserblog' ),
            'desc' => esc_html__( 'You can add maximum 1 field for the same taxonomy.', 'tmuserblog' ),
            'id'   => 'tax',
            'type' => 'select',
            'options'  => get_taxonomies(array('public' => true)),
            'default' => 'category'
        ) );

        $tax_metabox->add_group_field( $tax_field_id, array(
            'name' => esc_html__( 'Field Type', 'tmuserblog' ),
            'id'   => 'field_type',
            'type' => 'radio_inline',
                'options' => array(
                    'taxonomy_select' => esc_html__( 'Select Box', 'tmuserblog' ),
                    'taxonomy_multicheck' => esc_html__( 'Multi Select', 'tmuserblog' ),
                    'text' => esc_html__( 'Text Box', 'tmuserblog' )
                ),
                'default' => 'taxonomy_select',
        ) );

        $tax_metabox->add_group_field( $tax_field_id, array(
            'name' => esc_html__( 'Required Field', 'tmuserblog' ),
            'desc' => esc_html__( 'Make the field required.', 'tmuserblog' ),
            'id'   => 'required',
            'type' => 'radio_inline',
            'options'  => array(
                'enable' => esc_html__( 'Yes', 'tmuserblog' ),
                'disable' => esc_html__( 'No', 'tmuserblog' )
            ),
            'default' => 'disable'
        ) );

        /* Post Type Metabox */
        
        $post_type_metabox = new_cmb2_box( 
            array(
                'id' => 'tmuserb_post_type',
                'title' => esc_html__( 'Post Type', 'tmuserblog'),
                'object_types' => array('tmuserb'), // post type
                'context' => 'side',
                'priority' => 'high',
                'show_names' => false, // Show field names on the left
            ) 
        );

        $post_type_metabox->add_field(
            array(
                'name'    => esc_html__( 'Post Type', 'tmuserblog'), 
                'id'      => $prefix . '_post_type',
                'type' => 'select',
                'options' => get_post_types(array('public' => true)),
                'default' => 'post',
                'column' => array(
                    'position' => 2,
                    'name'     => esc_html__( 'Post Type', 'tmuserblog')
                )
            )
        );

        /* Recaptcha Metabox */
        
        $recaptcha_metabox = new_cmb2_box( 
            array(
                'id' => 'tmuserb_recaptcha_theme',
                'title' => esc_html__( 'Recaptcha Theme', 'tmuserblog'),
                'object_types' => array('tmuserb'), // post type
                'context' => 'side',
                'priority' => 'high',
                'show_names' => false, // Show field names on the left
            ) 
        );

        $recaptcha_metabox->add_field(
            array(
                'name'    => esc_html__( 'Recaptcha Theme', 'tmuserblog'),
                'id'      => $prefix . '_recaptcha_theme',
                'type' => 'radio_inline',
                'options' => array(
                    'light' => esc_html__( 'Light', 'tmuserblog'),
                    'dark' => esc_html__( 'Dark', 'tmuserblog')
                ),
                'default' => 'light'
            )
        );
    }

    /**
     * Used for modifying the column order
     */
    function array_insert( $array, $index, $insert ) {
        return array_slice( $array, 0, $index, true ) + $insert +
        array_slice( $array, $index, count( $array ) - $index, true);
    }

    /**
     * Add shortcode column to the admin table
     */
    public function add_admin_column($defaults){
        return $this->array_insert( $defaults, 2, [
            'tmuserb_shortcode' => esc_html__('Shortcode', 'tmuserblog')
        ]);
    }

    /**
     * Add shortcode column content
     */
    public function add_admin_column_content($column_name, $post_id){
        global $post;
        if($column_name === 'tmuserb_shortcode'){
            echo '<code>[tmuserb_form form_id="' . esc_attr($post_id) . '"]</code>';
        }
        return;
    }

    /**
     * Add admin styles
     */
    public function admin_styles() {
        echo '<style>#tmuserb_shortcode{width:300px}.wp-list-table tbody tr.status-pending th {border-left:3px solid #FF9800}#tmuserb_cmb2_post_type{width:120px}</style>';
    }
}

/**
 * Returns the main instance of TMuserBlogCPT.
 */

function TMuserBlogCPT() {  
	return TMuserBlogCPT::instance();
}
// Global for backwards compatibility.
$GLOBALS['TMuserBlogCPT'] = TMuserBlogCPT();