<?php
defined( 'ABSPATH' ) || exit;

class TMuserBlog {
    /* The single instance of the class */
	protected static $_instance = null;

    /* Check if scripts are required */
    protected static $add_scripts;

    /* Main Instance */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

    /* Constructor */
    public function __construct() {
        add_action('init', array($this, 'init'));
    }

    /* Init */
    public function init() {
		add_action( 'cmb2_init', array($this, 'form_register') );
        add_action( 'cmb2_init', array($this, 'edit_form_register') );
        add_shortcode( 'tmuserb_form', array($this, 'shortcode') );
        add_shortcode( 'tmuserb_blog', array($this, 'blog_shortcode') );
        add_action( 'cmb2_after_init', array($this, 'handle_submission') );
        /* We used "wp_footer" because we need to check if shortcode is used on the page before enqueue the scripts. */
        add_action( 'wp_footer', array($this, 'shortcode_enqueue_scripts') );
    }

    /**
     * Enqueue styles & scripts only if shortcode was added to the page
     */
    public function shortcode_enqueue_scripts() {

        if ( strpos($_SERVER['REQUEST_URI'], 'elementor') === false ) {
            if ( ! self::$add_scripts ) {
                return;
            }
        }

        // Get Custom Style Settings
        $custom_css = tmuserb_get_option('tmuserb_options_2','custom_css', '');
        $font_size = tmuserb_get_option('tmuserb_options_2','font_size', 15);
        $small_font_size = tmuserb_get_option('tmuserb_options_2','small_font_size', 14);
        $title_font_size = tmuserb_get_option('tmuserb_options_2','title_font_size', 24);
        $form_bg = tmuserb_get_option('tmuserb_options_2','form_bg', '#fafafa');
        $form_border_color = tmuserb_get_option('tmuserb_options_2','form_border_color', 'rgba(0,0,0,0.1)');
        $title_color = tmuserb_get_option('tmuserb_options_2','title_color', '#000000');
        $default_color = tmuserb_get_option('tmuserb_options_2','default_color', '#666666');
        $link_color = tmuserb_get_option('tmuserb_options_2','link_color', '#000000');
        $link_hover_color = tmuserb_get_option('tmuserb_options_2','link_hover_color', '#077CB2');
        $form_field_bg = tmuserb_get_option('tmuserb_options_2','form_field_bg', '#ffffff');
        $form_field_color = tmuserb_get_option('tmuserb_options_2','form_field_color', '#000000');
        $form_field_border_color = tmuserb_get_option('tmuserb_options_2','form_field_border_color', 'rgba(0,0,0,0.15)');
        $form_field_focus_border_color = tmuserb_get_option('tmuserb_options_2','form_field_focus_border_color', '#000000');
        $table_border_color = tmuserb_get_option('tmuserb_options_2','table_border_color', 'rgba(0,0,0,0.1)');
        $table_bg = tmuserb_get_option('tmuserb_options_2','table_bg', '#ffffff');
        $table_color = tmuserb_get_option('tmuserb_options_2','table_color', '#666666');
        $table_title_color = tmuserb_get_option('tmuserb_options_2','table_title_color', '#000000');
        $table_pg_bg = tmuserb_get_option('tmuserb_options_2','table_pg_bg', '#ffffff');
        $table_pg_color = tmuserb_get_option('tmuserb_options_2','table_pg_color', '#000000');
        $table_pg_bg_hover = tmuserb_get_option('tmuserb_options_2','table_pg_bg_hover', '#000000');
        $table_pg_color_hover = tmuserb_get_option('tmuserb_options_2','table_pg_color_hover', '#ffffff');
        $info_color = tmuserb_get_option('tmuserb_options_2','info_color', '#2196F3');
        $error_color = tmuserb_get_option('tmuserb_options_2','error_color', '#F44336');
        $warning_color = tmuserb_get_option('tmuserb_options_2','warning_color', '#FF9800');
        $success_color = tmuserb_get_option('tmuserb_options_2','success_color', '#009688');
        $button_bg = tmuserb_get_option('tmuserb_options_2','button_bg', '#000000');
        $button_color = tmuserb_get_option('tmuserb_options_2','button_color', '#ffffff');
        $button_bg_hover = tmuserb_get_option('tmuserb_options_2','button_bg_hover', '#077CB2');
        $button_color_hover = tmuserb_get_option('tmuserb_options_2','button_color_hover', '#ffffff');

        // Styles
        wp_enqueue_style('tmuserb-table', plugin_dir_url( __FILE__ ) . 'css/table.min.css', false, '1.0');
        wp_enqueue_style('tmuserb-form', plugin_dir_url( __FILE__ ) . 'css/form.css', false, '1.0.9');
        if (is_rtl()) {
            wp_enqueue_style('tmuserb-form-rtl', plugin_dir_url( __FILE__ ) . 'css/form-rtl.css', 'tmuserb-table', '1.0');
        }

        // Custom Styles
        $inline_style = '';

        if ((!empty($font_size)) && ($font_size != 15)) {
            $inline_style .= '#tmuserb-content,div[id^=cmb2-metabox-tmuseb-front-end-post-form],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="date"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="password"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="search"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="tel"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] select,div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea,.dataTables_wrapper .dataTables_filter input,#tmuserb_login_form input[type="text"],#tmuserb_login_form input[type="password"],.tmuserb-tab-content .cmb-form input[type="submit"],#tmuserb_login_form input[type="submit"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] .cmb-row,div[id^=cmb2-metabox-tmuseb-front-end-post-form] .wp-picker-input-wrap input[type="button"],.embed-status .cmb2-remove-wrapper,.tmuserb-notice p,.tmuserb-table-container button.tmuserb-table-btn {font-size:' . $font_size . 'px}';
        }

        if ((!empty($small_font_size)) && ($small_font_size != 14)) {
            $inline_style .= 'div[id^=cmb2-metabox-tmuseb-front-end-post-form] .cmb2-metabox-description,.ui-state-error-text,.tmuserb-table,#tmuserb-table_info {font-size:' . $small_font_size . 'px}';
        }

        if ((!empty($title_font_size)) && ($title_font_size != 24)) {
            $inline_style .= '#tmuserb-content .tmuserb-edit-form-title {font-size:' . $title_font_size . 'px}';
        }

        if ((!empty($default_color)) && ($default_color != '#666666')) {
            $inline_style .= '.tmuserb-tab-content,#tmuserb-table_info,.tmuserb-tab-content p {color:' . $default_color . '}';
        }

        if ((!empty($title_color)) && ($title_color != '#000000')) {
            $inline_style .= '#tmuserb-content .tmuserb-edit-form-title,div[id^=cmb2-metabox-tmuseb-front-end-post-form] .cmb-row .cmb-th label,#tmuserb-tabs li a.tmuserb-tab,#tmuserb-tabs li a.tmuserb-tab.active,#tmuserb-tabs li a.tmuserb-tab:hover,#tmuserb_login_form label {color:' . $title_color . '}';
        }

        if ((!empty($link_color)) && ($link_color != '#000000')) {
            $inline_style .= '#tmuserb-content .tmuserb-edit-form-title .tmuserb-add-post,.embed-status .cmb2-remove-wrapper a,.tmuserb-table a,.tmuserb-table-container button.tmuserb-table-btn {color:' . $link_color . '}';
        }

        if ((!empty($link_hover_color)) && ($link_hover_color != '#077CB2')) {
            $inline_style .= '#tmuserb-content .tmuserb-edit-form-title .tmuserb-add-post:hover,.tmuserb-table a:hover,.tmuserb-table-container button.tmuserb-table-btn.active,.tmuserb-table-container button.tmuserb-table-btn:hover {color:' . $link_color . '}';
        }

        if ((!empty($form_bg)) && ($form_bg != '#fafafa')) {
            $inline_style .= '#tmuserb-tabs li a.tmuserb-tab.active,#tmuserb-tabs li a.tmuserb-tab:hover,.tmuserb-tab-content {background:' . $form_bg . '}';
        }

        if ((!empty($form_border_color)) && ($form_border_color != 'rgba(0,0,0,0.1)')) {
            $inline_style .= '.tmuserb-login-form-container,#tmuserb-tabs,#tmuserb-tabs li a.tmuserb-tab.active,#tmuserb-tabs li a.tmuserb-tab:hover,.tmuserb-tab-content,.dataTables_wrapper .dataTables_paginate .paginate_button,.cmb2-id-submitted-draft {border-color:' . $form_border_color . '}';
        }

        if ((!empty($form_field_bg)) && ($form_field_bg != '#ffffff')) {
            $inline_style .= 'div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] select,div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea,.dataTables_wrapper .dataTables_filter input,#tmuserb_login_form input[type="text"],#tmuserb_login_form input[type="password"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"]:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"]:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"]:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"]:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] select:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea:focus,.dataTables_wrapper .dataTables_filter input:focus,#tmuserb_login_form input[type="text"]:focus,#tmuserb_login_form input[type="password"]:focus {background-color:' . $form_bg . '}';
        }

        if ((!empty($form_field_color)) && ($form_field_color != '#000000')) {
            $inline_style .= 'div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] select,div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea,.dataTables_wrapper .dataTables_filter input,#tmuserb_login_form input[type="text"],#tmuserb_login_form input[type="password"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"]:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"]:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"]:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"]:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] select:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea:focus,.dataTables_wrapper .dataTables_filter input:focus,#tmuserb_login_form input[type="text"]:focus,#tmuserb_login_form input[type="password"]:focus {color:' . $form_field_color . '}';
        }

        if ((!empty($form_field_border_color)) && ($form_field_border_color != 'rgba(0,0,0,0.15)')) {
            $inline_style .= 'div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] select,div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea,.dataTables_wrapper .dataTables_filter input,#tmuserb_login_form input[type="text"],#tmuserb_login_form input[type="password"] {border-color:' . $form_field_border_color . '}';
        }

        if ((!empty($form_field_focus_border_color)) && ($form_field_focus_border_color != '#000000')) {
            $inline_style .= 'div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"]:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"]:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"]:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"]:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] select:focus,div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea:focus,.dataTables_wrapper .dataTables_filter input:focus,#tmuserb_login_form input[type="text"]:focus,#tmuserb_login_form input[type="password"]:focus {border-color:' . $form_field_focus_border_color . '}';
        }

        if ((!empty($table_border_color)) && ($table_border_color != 'rgba(0,0,0,0.1)')) {
            $inline_style .= 'table.dataTable.cell-border tbody th,table.dataTable.cell-border tbody td,table.dataTable thead th,table.dataTable thead td,.dataTables_wrapper.no-footer .dataTables_scrollBody {border-color:' . $table_border_color . ' !important}';
        }

        if ((!empty($table_bg)) && ($table_bg != '#ffffff')) {
            $inline_style .= 'table.dataTable.cell-border tbody th,table.dataTable.cell-border tbody td,table.dataTable thead th,table.dataTable thead td,.dataTables_wrapper.no-footer .dataTables_scrollBody {background:' . $table_bg . ' !important}';
        }

        if ((!empty($table_color)) && ($table_color != '#666666')) {
            $inline_style .= 'table.dataTable tbody td {color:' . $table_color . '}';
        }

        if ((!empty($table_title_color)) && ($table_title_color != '#000000')) {
            $inline_style .= 'table.dataTable thead th {color:' . $table_title_color . '}';
        }

        if ((!empty($table_pg_bg)) && ($table_pg_bg != '#ffffff')) {
            $inline_style .= '.dataTables_wrapper .dataTables_paginate .paginate_button {background:' . $table_pg_bg . '}';
        }

        if ((!empty($table_pg_color)) && ($table_pg_color != '#000000')) {
            $inline_style .= '.dataTables_wrapper .dataTables_paginate .paginate_button {color:' . $table_pg_color . '}';
        }

        if ((!empty($table_pg_bg_hover)) && ($table_pg_bg_hover != '#000000')) {
            $inline_style .= '.dataTables_wrapper .dataTables_paginate .paginate_button.current, .dataTables_wrapper .dataTables_paginate .paginate_button.current:hover, .dataTables_wrapper .dataTables_paginate .paginate_button:active, .dataTables_wrapper .dataTables_paginate .paginate_button:hover {background:' . $table_pg_bg_hover . ';border-color:' . $table_pg_bg_hover . '}';
        }

        if ((!empty($table_pg_color_hover)) && ($table_pg_color_hover != '#ffffff')) {
            $inline_style .= '.dataTables_wrapper .dataTables_paginate .paginate_button.current, .dataTables_wrapper .dataTables_paginate .paginate_button.current:hover, .dataTables_wrapper .dataTables_paginate .paginate_button:active, .dataTables_wrapper .dataTables_paginate .paginate_button:hover {color:' . $table_pg_color_hover . '}';
        }

        if ((!empty($info_color)) && ($info_color != '#2196F3')) {
            $inline_style .= '.tmuserb-notice.info p {background-color:' . $info_color . '}';
            $inline_style .= 'table.dataTable tbody td.post_private {color:' . $info_color . '}';
        }

        if ((!empty($error_color)) && ($error_color != '#F44336')) {
            $inline_style .= '.tmuserb-notice.error p {background-color:' . $error_color . '}';
            $inline_style .= '.embed-status .cmb2-remove-wrapper a:hover,div[id^="cmb2-metabox-tmuseb-front-end-post-form"] .cmb-row .cmb-th label.required-field,table.dataTable tbody td.post_pending,table.dataTable tbody td .tmuserb-icon-delete,table.dataTable tbody td.dataTables_empty {color:' . $info_color . '}';
        }

        if ((!empty($warning_color)) && ($warning_color != '#FF9800')) {
            $inline_style .= '.tmuserb-notice.warning p {background-color:' . $warning_color . '}';
            $inline_style .= 'table.dataTable tbody td.post_draft,div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput .tag .tag-remove {color:' . $info_color . '}';
        }

        if ((!empty($success_color)) && ($success_color != '#009688')) {
            $inline_style .= '.tmuserb-notice.success p {background-color:' . $success_color . '}';
            $inline_style .= 'table.dataTable tbody td.post_publish,table.dataTable tbody td .tmuserb-icon-edit {color:' . $info_color . '}';
        }

        if ((!empty($button_bg)) && ($button_bg != '#000000')) {
            $inline_style .= '.tmuserb-tab-content .cmb-form input[type="submit"],#tmuserb_login_form input[type="submit"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput .tag {background:' . $button_bg . '}';
        }

        if ((!empty($button_color)) && ($button_color != '#ffffff')) {
            $inline_style .= '.tmuserb-tab-content .cmb-form input[type="submit"],#tmuserb_login_form input[type="submit"],div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput .tag {color:' . $button_color . '}';
        }

        if ((!empty($button_bg_hover)) && ($button_bg_hover != '#077CB2')) {
            $inline_style .= '.tmuserb-tab-content .cmb-form input[type="submit"]:hover,.tmuserb-tab-content .cmb-form input[type="submit"]:focus,#tmuserb_login_form input[type="submit"]:hover,#tmuserb_login_form input[type="submit"]:focus {background:' . $button_bg_hover . '}';
        }

        if ((!empty($button_color_hover)) && ($button_color_hover != '#ffffff')) {
            $inline_style .= '.tmuserb-tab-content .cmb-form input[type="submit"]:hover,.tmuserb-tab-content .cmb-form input[type="submit"]:focus,#tmuserb_login_form input[type="submit"]:hover,#tmuserb_login_form input[type="submit"]:focus {color:' . $button_color_hover . '}';
        }

        if (!empty($custom_css)) {
            $inline_style .= $custom_css;
        }

        wp_add_inline_style( 'tmuserb-form', $inline_style );

        // Scripts
        wp_enqueue_script( 'tmuserb-table', plugins_url( '/js/table.min.js', __FILE__), array( 'jquery' ), '1.0', true ); 
        wp_enqueue_script( 'tmuserb-tags', plugins_url( '/js/tags.min.js', __FILE__), array( 'jquery' ), '1.0', true ); 
        wp_enqueue_script( 'tmuserb-form', plugins_url( '/js/form.js', __FILE__), array( 'jquery' ), '1.0.5', true ); 
        if (!empty($this->get_site_key()) && !empty($this->get_secret_key())) {
            wp_enqueue_script( 'recaptcha', '//www.google.com/recaptcha/api.js', '', '1.0', true ); 
        }
        $table_param = array(
			"lengthMenu" => esc_html__( 'Display _MENU_ records per page', 'tmuserblog' ),
			"zeroRecords" => esc_html__( 'Nothing found - sorry', 'tmuserblog' ),
			"info" => esc_html__( 'Showing page _PAGE_ of _PAGES_', 'tmuserblog' ),
			"infoEmpty" => esc_html__( 'No records available', 'tmuserblog' ),
			"infoFiltered" => esc_html__( '(filtered from _MAX_ total records)', 'tmuserblog' ),
			"searchPlaceholder" => esc_html__( 'Search...', 'tmuserblog' ),
			"processing" => esc_html__( 'Processing...', 'tmuserblog' ),
            "all" => esc_html__( 'All', 'tmuserblog' ),
            "publish" => esc_html__( 'Published', 'tmuserblog' ),
            "pending" => esc_html__( 'Pending Review', 'tmuserblog' ),
            "pending_2" => esc_html__( 'Pending', 'tmuserblog' ),
            "draft" => esc_html__( 'Draft', 'tmuserblog' ),
            "pageLength" => esc_html(tmuserb_get_option('tmuserb_options_1','page_length', 20))
		);
		wp_localize_script('tmuserb-form', 'tmuserb_table_vars', $table_param);
    }

    /**
     * Register the edit form
     */
    public function edit_form_register() {
        $post_id = '';
        if (isset($_GET['post_edit'])) {
            if ($_GET['post_edit']) {
                $post_id = $_GET['post_edit'];
            } else {
                die();
            }
        }
        $title_length = tmuserb_get_option('tmuserb_options_1','max_title_length', 60);
        $excerpt_length = tmuserb_get_option('tmuserb_options_1','max_excerpt_length', 120);
        $teeny = tmuserb_get_option('tmuserb_options_1','teeny', 'disable');
        $quicktags = tmuserb_get_option('tmuserb_options_1','quicktags', 'disable');
        $media_upload = tmuserb_get_option('tmuserb_options_1','media_upload', 'disable');

        if (!current_user_can('upload_files')) {
            $media_upload = 'disable';
        }

        $wp_query = new WP_Query(
            array('post_type' => 'tmuserb', 'posts_per_page' => 99, 'post_status' => 'publish') 
        );

        while($wp_query->have_posts()) : $wp_query->the_post();

        // Create Meta Box

        $cmb = new_cmb2_box( array(
            'id'           => 'tmuseb-front-end-post-form-edit-' . get_the_id(),
            'object_types' => array( get_post_meta( get_the_id(), 'tmuserb_cmb2_post_type', true ) ),
            'hookup'       => false,
            'save_fields'  => false,
            'cmb_styles' => false
        ) );

        // Post Title

        $cmb->add_field( array(
            'name'    => esc_html__( 'Title:', 'tmuserblog' ),
            'id'      => 'submitted_post_title',
            'type'    => 'text',
            'default' => esc_html(get_the_title($post_id)),
            'attributes' => array(
                'class' => $this->get_form_class(),
                'autocomplete' => 'off',
                'maxlength' => $title_length,
                'required' => 'required'
            ),
        ));

        // Default Fields

        $default_fields = get_post_meta( get_the_id(), 'tmuserb_cmb2_defaults', true );
        $req_main = get_post_meta( get_the_id(), 'tmuserb_cmb2_req_main', true );

        if (in_array('post_format', $default_fields, false)) {
            $cmb->add_field( array(
                'name'    => esc_html__( 'Post Format:', 'tmuserblog' ),
                'id'      => 'submitted_post_format',
                'type'    => 'radio',
                'default'   => get_post_format(get_the_id()),
                'options' => $this->get_post_formats()
            ));
        }

        // Content
        if (in_array('content', $default_fields, false)) {
            $cmb->add_field( array(
                'name'       => esc_html__( 'Content:', 'tmuserblog' ),
                'id'         => 'submitted_post_content',
                'type'       => 'wysiwyg',
                'default'   => wp_kses_post(get_the_content(null, false, $post_id)),
                'options'    => array(
                    'teeny' => ($teeny == 'disable') ? false : true,
                    'quicktags' => ($quicktags == 'disable') ? false : true,
                    'textarea_rows' => 12,
                    'media_buttons' => ($media_upload == 'disable') ? false : true,
                ),
            ));
        }

        // Excerpt
        if (in_array('excerpt', $default_fields, false)) {
            $cmb->add_field( array(
                'name'    => esc_html__( 'Excerpt:', 'tmuserblog' ),
                'id'      => 'submitted_post_excerpt',
                'type'    => 'textarea_small',
                'default'   => esc_html(get_the_excerpt($post_id)),
                'attributes' => array(
                    'class' => $this->get_form_class(),
                    'autocomplete' => 'off',
                    'maxlength' => $excerpt_length
                )
            ));
        }

        // Featured image
        if (in_array('featured', $default_fields, false)) {
            $featured_attr = array(
                'class' => 'form-control-file',
                'type' => 'file',
                'accept' => 'image/*'
            );
            if (is_array($req_main)) {
                if (in_array('featured', $req_main, false)) {
                    $featured_attr['required'] = 'required';
                }
            }
            $cmb->add_field( array(
                'name'       => esc_html__( 'Featured Image:', 'tmuserblog' ),
                'id'         => 'submitted_post_thumbnail',
                'type'       => 'text',
                'after_field' => (has_post_thumbnail($post_id)) ? get_the_post_thumbnail($post_id, 'medium') : '',
                'attributes' => $featured_attr
            ) );
        }

        // Additional Fields

        $additional_fields = get_post_meta( get_the_id(), 'tmuserb_cmb2_additional_fields', true );
        $custom_fields = get_post_meta( get_the_id(), 'tmuserb_cmb2_custom_fields', true );
        $required_fields = array();

        if (!empty($custom_fields) && $additional_fields == 'enable') {
            foreach ( (array) $custom_fields as $key => $entry ) { 
                $label = $meta_key = $default_value = $desc = $field_type = $field_options = '';
                if ( isset( $entry['label'] ) ) {            
                    $label = $entry['label'];
                }
                if ( isset( $entry['default_value'] ) ) {            
                    $default_value = $entry['default_value'];
                }
                if ( isset( $entry['desc'] ) ) {            
                    $desc = $entry['desc'];
                }
                if ( isset( $entry['meta_key'] ) ) {            
                    $meta_key = $entry['meta_key'];
                    $default_value = get_post_meta( $post_id, $meta_key, true );
                }
                if ( isset( $entry['field_type'] ) ) {            
                    $field_type = $entry['field_type'];
                }
                if ( isset( $entry['field_options'] ) ) {            
                    $field_options = $entry['field_options'];
                }
                if ( isset( $entry['meta_key'] ) && isset( $entry['required'] ) && $entry['required'] == 'enable' ) {
                    array_push($required_fields, 'post_meta_' . $meta_key);
                }
                // Rich text editor
                if ($field_type == 'wysiwyg') {
                    $cmb->add_field( array(
                        'name'    => $label,
                        'desc' => $desc,
                        'id'      => 'post_meta_' . $meta_key,
                        'type'    => 'wysiwyg',
                        'default' => $default_value,
                        'options'    => array(
                            'teeny' => ($teeny == 'disable') ? false : true,
                            'quicktags' => ($quicktags == 'disable') ? false : true,
                            'textarea_rows' => 12,
                            'media_buttons' => ($media_upload == 'disable') ? false : true,
                        )
                    ));
                } 
                // Number
                else if ($field_type == 'number') {
                    $cmb->add_field( array(
                        'name'    => $label,
                        'desc' => $desc,
                        'id'      => 'post_meta_' . $meta_key,
                        'type'    => 'text',
                        'default' => $default_value,
                        'attributes' => array(
                            'class' => $this->get_form_class(),
                            'type' => 'number',
		                    'pattern' => '\d*',
                            'autocomplete' => 'off'
                        )
                    ));
                } 
                // Colorpicker
                else if ($field_type == 'colorpicker') {
                    $cmb->add_field( array(
                        'name'    => $label,
                        'desc' => $desc,
                        'id'      => 'post_meta_' . $meta_key,
                        'type'    => 'colorpicker',
                        'default' => $default_value
                    ));
                } 
                // Oembed
                else if ($field_type == 'oembed') {
                    $cmb->add_field( array(
                        'name'    => $label,
                        'desc' => $desc,
                        'id'      => 'post_meta_' . $meta_key,
                        'type'    => 'oembed',
                        'default' => $default_value,
                        'attributes' => array(
                            'class' => $this->get_form_class() . ' cmb2-oembed',
                            'autocomplete' => 'off'
                        )
                    ));
                } 
                // Title
                else if ($field_type == 'title') {
                    $cmb->add_field( array(
                        'name'    => $label,
                        'desc' => $desc,
                        'id'      => 'title_' . rand(),
                        'type'    => 'title'
                    ));
                } 
                // Select & Radio
                else if ($field_type == 'select' || $field_type == 'radio') {
                    $cmb->add_field( array(
                        'name'    => $label,
                        'desc' => $desc,
                        'id'      => 'post_meta_' . $meta_key,
                        'type'    => $field_type,
                        'show_option_none' => false,
                        'default' => $default_value,
                        'options' => $this->get_field_options($field_options),
                        'attributes' => array(
                            'autocomplete' => 'off'
                        )
                    ));
                } 
                // Other fields
                else {
                    $cmb->add_field( array(
                        'name'    => $label,
                        'desc' => $desc,
                        'id'      => 'post_meta_' . $meta_key,
                        'type'    => $field_type,
                        'default' => $default_value,
                        'attributes' => array(
                            'class' => $this->get_form_class(),
                            'autocomplete' => 'off'
                        )
                    ));
                }
            }
        }

        // Taxonomies

        $cmb2_taxonomies = get_post_meta( get_the_id(), 'tmuserb_cmb2_taxonomies', true );
        $cmb2_tax = get_post_meta( get_the_id(), 'tmuserb_cmb2_tax', true );

        if (!empty($cmb2_tax) && $cmb2_taxonomies == 'enable') {
            $max_tax = tmuserb_get_option('tmuserb_options_1','max_tax', 60);
            foreach ( (array) $cmb2_tax as $key => $entry ) { 
                $label = '';
                $tax = 'category';
                $field_type = 'taxonomy_select';
                if ( isset( $entry['label'] ) ) {            
                    $label = $entry['label'];
                }
                if ( isset( $entry['tax'] ) ) {            
                    $tax = $entry['tax'];
                }
                if ( isset( $entry['field_type'] ) ) {            
                    $field_type = $entry['field_type'];
                }

                if ($field_type == 'taxonomy_select') {
                    $cmb->add_field( array(
                        'name'       => $label,
                        'id'         => 'post_tax_' . $tax,
                        'type'       => $field_type,
                        'taxonomy'   => $tax,
                        'query_args' => array(
                            'number' => $max_tax
                        ),
                    ) );
                    if ( isset( $entry['required'] ) && $entry['required'] == 'enable' ) {
                        array_push($required_fields, 'post_tax_' . $tax);
                    }
                } else if ($field_type == 'taxonomy_multicheck') {
                    $cmb->add_field( array(
                        'name'       => $label,
                        'id'         => 'post_tax_' . $tax,
                        'type'       => $field_type,
                        'select_all_button' => false,
                        'taxonomy'   => $tax,
                        'query_args' => array(
                            'number' => $max_tax
                        ),
                    ) );
                    if ( isset( $entry['required'] ) && $entry['required'] == 'enable' ) {
                        array_push($required_fields, 'post_tax_' . $tax);
                    }
                } else {
                    $cmb->add_field( array(
                        'name' => $label,
                        'id' => 'custom_post_tax_' . $tax,
                        'type' => 'text',
                        'attributes' => array(
                            'autocomplete' => 'off',
                            'class' => 'tmuserb-tags'
                        )
                    ) );
                    if ( isset( $entry['required'] ) && $entry['required'] == 'enable' ) {
                        array_push($required_fields, 'custom_post_tax_' . $tax);
                    }
                }
                if ( isset( $entry['meta_key'] ) && isset( $entry['required'] ) && $entry['required'] == 'enable' ) {
                    array_push($required_fields, 'post_meta_' . $meta_key);
                }
            }
        }

        // Store required field ids

        if (is_array($req_main)) {
            $main_fields = array(
                'post_format' => 'submitted_post_format',
                'content' => 'submitted_post_content',
                'excerpt' => 'submitted_post_excerpt',
                'featured' => 'submitted_post_thumbnail',
            );

            foreach ( $default_fields as $key => $entry ) { 
                if (in_array($entry, $req_main, false)) {
                    array_push($required_fields, $main_fields[$entry]);
                }
            }
        }

        $get_required_fields = implode(",", $required_fields);

        $cmb->add_field( array(
            'id'      => 'required_fields',
            'type'    => 'hidden',
            'save_field' => false,
            'default' => $get_required_fields,
            'attributes' => array(
                'autocomplete' => 'off'
            )
        ));

        $default_draft = get_post_meta( $post_id, 'submitted_draft', true );
        if (empty($default_draft)) {
            $default_draft = 'no';
        }

        $recaptcha_theme = get_post_meta( get_the_id(), 'tmuserb_cmb2_recaptcha_theme', true );
        if (empty($recaptcha_theme)) {
            $recaptcha_theme = 'light';
        }

        $cmb->add_field( array(
            'name'    => esc_html__( 'Save as draft?', 'tmuserblog' ),
            'id'      => 'submitted_draft',
            'show_option_none' => false,
            'type' => 'radio',
            'after_row' => array($this, 'recaptcha'),
            'options'          => array(
                'yes' => esc_html__( 'Yes', 'tmuserblog' ),
                'no' => esc_html__( 'No', 'tmuserblog' ),
            ),
            'attributes' => array(
                'data-theme' => $recaptcha_theme
            ),
            'default' => $default_draft
        ));

        endwhile;

        wp_reset_postdata();

    }

    /**
     * Gets the front-end-edit-form cmb instance
     *
     * @return CMB2 object
     */
    public function get_edit_form($form_id, $object_id) {
        // Use ID of metabox in form_register
        $metabox_id = 'tmuseb-front-end-post-form-edit-' . $form_id;

        // Get CMB2 metabox object
        $author_id = get_post_field( 'post_author', $object_id );
        $user_id = get_current_user_id();
        if ($author_id == $user_id) {
            return cmb2_get_metabox( $metabox_id, $object_id );
        } else {
            wp_die(esc_html__( 'You are not allowed to update this post!', 'tmuserblog' ));
        }
    }

    /**
     * Register the form and fields for our front-end submission form
     */
    public function form_register() {
        $title_length = tmuserb_get_option('tmuserb_options_1','max_title_length', 60);
        $excerpt_length = tmuserb_get_option('tmuserb_options_1','max_excerpt_length', 120);
        $teeny = tmuserb_get_option('tmuserb_options_1','teeny', 'disable');
        $quicktags = tmuserb_get_option('tmuserb_options_1','quicktags', 'disable');
        $media_upload = tmuserb_get_option('tmuserb_options_1','media_upload', 'disable');

        if (!current_user_can('upload_files')) {
            $media_upload = 'disable';
        }

        $wp_query = new WP_Query(
            array('post_type' => 'tmuserb', 'posts_per_page' => 99, 'post_status' => 'publish') 
        );

        while($wp_query->have_posts()) : $wp_query->the_post();

        // Create Meta Box

        $cmb = new_cmb2_box( array(
            'id'           => 'tmuseb-front-end-post-form-' . get_the_id(),
            'object_types' => array( get_post_meta( get_the_id(), 'tmuserb_cmb2_post_type', true ) ),
            'hookup'       => false,
            'save_fields'  => false,
            'cmb_styles' => false
        ) );

        // Post Title

        $cmb->add_field( array(
            'name'    => esc_html__( 'Title:', 'tmuserblog' ),
            'id'      => 'submitted_post_title',
            'type'    => 'text',
            'default' => esc_html__( 'New Post', 'tmuserblog' ),
            'attributes' => array(
                'class' => $this->get_form_class(),
                'autocomplete' => 'off',
                'maxlength' => $title_length,
                'required' => 'required'
            ),
        ));

        // Default Fields

        $default_fields = get_post_meta( get_the_id(), 'tmuserb_cmb2_defaults', true );
        $req_main = get_post_meta( get_the_id(), 'tmuserb_cmb2_req_main', true );

        if (in_array('post_format', $default_fields, false)) {
            $cmb->add_field( array(
                'name'    => esc_html__( 'Post Format:', 'tmuserblog' ),
                'id'      => 'submitted_post_format',
                'type'    => 'radio',
                'default'   => 'standard',
                'options' => $this->get_post_formats()
            ));
        }

        // Content
        if (in_array('content', $default_fields, false)) {
            $cmb->add_field( array(
                'name'       => esc_html__( 'Content:', 'tmuserblog' ),
                'id'         => 'submitted_post_content',
                'type'       => 'wysiwyg',
                'options'    => array(
                    'teeny' => ($teeny == 'disable') ? false : true,
                    'quicktags' => ($quicktags == 'disable') ? false : true,
                    'textarea_rows' => 12,
                    'media_buttons' => ($media_upload == 'disable') ? false : true,
                ),
            ));
        }

        // Excerpt
        if (in_array('excerpt', $default_fields, false)) {
            $cmb->add_field( array(
                'name'    => esc_html__( 'Excerpt:', 'tmuserblog' ),
                'id'      => 'submitted_post_excerpt',
                'type'    => 'textarea_small',
                'attributes' => array(
                    'class' => $this->get_form_class(),
                    'autocomplete' => 'off',
                    'maxlength' => $excerpt_length
                )
            ));
        }

        // Featured image
        if (in_array('featured', $default_fields, false)) {
            $featured_attr = array(
                'class' => 'form-control-file',
                'type' => 'file',
                'accept' => 'image/*'
            );
            if (is_array($req_main)) {
                if (in_array('featured', $req_main, false)) {
                    $featured_attr['required'] = 'required';
                }
            }
            $cmb->add_field( array(
                'name'       => esc_html__( 'Featured Image:', 'tmuserblog' ),
                'id'         => 'submitted_post_thumbnail',
                'type'       => 'text',
                'attributes' => $featured_attr
            ) );
        }

        // Additional Fields

        $additional_fields = get_post_meta( get_the_id(), 'tmuserb_cmb2_additional_fields', true );
        $custom_fields = get_post_meta( get_the_id(), 'tmuserb_cmb2_custom_fields', true );
        $required_fields = array();

        if (!empty($custom_fields) && $additional_fields == 'enable') {
            foreach ( (array) $custom_fields as $key => $entry ) { 
                $label = $meta_key = $default_value = $desc = $field_type = $field_options = '';
                if ( isset( $entry['label'] ) ) {            
                    $label = $entry['label'];
                }
                if ( isset( $entry['meta_key'] ) ) {            
                    $meta_key = $entry['meta_key'];
                }
                if ( isset( $entry['default_value'] ) ) {            
                    $default_value = $entry['default_value'];
                }
                if ( isset( $entry['desc'] ) ) {            
                    $desc = $entry['desc'];
                }
                if ( isset( $entry['field_type'] ) ) {            
                    $field_type = $entry['field_type'];
                }
                if ( isset( $entry['field_options'] ) ) {            
                    $field_options = $entry['field_options'];
                }
                if ( isset( $entry['meta_key'] ) && isset( $entry['required'] ) && $entry['required'] == 'enable' ) {
                    array_push($required_fields, 'post_meta_' . $meta_key);
                }
                // Rich Text Editor
                if ($field_type == 'wysiwyg') {
                    $cmb->add_field( array(
                        'name'    => $label,
                        'desc' => $desc,
                        'id'      => 'post_meta_' . $meta_key,
                        'type'    => 'wysiwyg',
                        'default' => $default_value,
                        'options'    => array(
                            'teeny' => ($teeny == 'disable') ? false : true,
                            'quicktags' => ($quicktags == 'disable') ? false : true,
                            'textarea_rows' => 12,
                            'media_buttons' => ($media_upload == 'disable') ? false : true,
                        )
                    ));
                } 
                // Number
                else if ($field_type == 'number') {
                    $cmb->add_field( array(
                        'name'    => $label,
                        'desc' => $desc,
                        'id'      => 'post_meta_' . $meta_key,
                        'type'    => 'text',
                        'default' => $default_value,
                        'attributes' => array(
                            'class' => $this->get_form_class(),
                            'type' => 'number',
		                    'pattern' => '\d*',
                            'autocomplete' => 'off'
                        )
                    ));
                } 
                // Colorpicker
                else if ($field_type == 'colorpicker') {
                    $cmb->add_field( array(
                        'name'    => $label,
                        'desc' => $desc,
                        'id'      => 'post_meta_' . $meta_key,
                        'type'    => 'colorpicker',
                        'default' => $default_value
                    ));
                } 
                // Oembed
                else if ($field_type == 'oembed') {
                    $cmb->add_field( array(
                        'name'    => $label,
                        'desc' => $desc,
                        'id'      => 'post_meta_' . $meta_key,
                        'type'    => 'oembed',
                        'default' => $default_value,
                        'attributes' => array(
                            'class' => $this->get_form_class() . ' cmb2-oembed',
                            'autocomplete' => 'off'
                        )
                    ));
                } 
                // Title
                else if ($field_type == 'title') {
                    $cmb->add_field( array(
                        'name'    => $label,
                        'desc' => $desc,
                        'id'      => 'title_' . rand(),
                        'type'    => 'title'
                    ));
                }
                // Select & Radio
                else if ($field_type == 'select' || $field_type == 'radio') {
                    $cmb->add_field( array(
                        'name'    => $label,
                        'desc' => $desc,
                        'id'      => 'post_meta_' . $meta_key,
                        'type'    => $field_type,
                        'show_option_none' => false,
                        'default' => $default_value,
                        'options' => $this->get_field_options($field_options)
                    ));
                }
                // Other Fields
                else {
                    $cmb->add_field( array(
                        'name'    => $label,
                        'desc' => $desc,
                        'id'      => 'post_meta_' . $meta_key,
                        'type'    => $field_type,
                        'default' => $default_value,
                        'attributes' => array(
                            'class' => $this->get_form_class()
                        )
                    ));
                }
            }
        }

        // Taxonomies

        $cmb2_taxonomies = get_post_meta( get_the_id(), 'tmuserb_cmb2_taxonomies', true );
        $cmb2_tax = get_post_meta( get_the_id(), 'tmuserb_cmb2_tax', true );

        if (!empty($cmb2_tax) && $cmb2_taxonomies == 'enable') {
            $max_tax = tmuserb_get_option('tmuserb_options_1','max_tax', 60);
            foreach ( (array) $cmb2_tax as $key => $entry ) { 
                $label = '';
                $tax = 'category';
                $field_type = 'taxonomy_select';
                if ( isset( $entry['label'] ) ) {            
                    $label = $entry['label'];
                }
                if ( isset( $entry['tax'] ) ) {            
                    $tax = $entry['tax'];
                }
                if ( isset( $entry['field_type'] ) ) {            
                    $field_type = $entry['field_type'];
                }

                if ($field_type == 'taxonomy_select') {
                    $cmb->add_field( array(
                        'name'       => $label,
                        'id'         => 'post_tax_' . $tax,
                        'type'       => $field_type,
                        'taxonomy'   => $tax,
                        'query_args' => array(
                            'number' => $max_tax
                        ),
                    ) );
                    if ( isset( $entry['required'] ) && $entry['required'] == 'enable' ) {
                        array_push($required_fields, 'post_tax_' . $tax);
                    }
                } else if ($field_type == 'taxonomy_multicheck') {
                    $cmb->add_field( array(
                        'name'       => $label,
                        'id'         => 'post_tax_' . $tax,
                        'type'       => $field_type,
                        'select_all_button' => false,
                        'taxonomy'   => $tax,
                        'query_args' => array(
                            'number' => $max_tax
                        ),
                    ) );
                    if ( isset( $entry['required'] ) && $entry['required'] == 'enable' ) {
                        array_push($required_fields, 'post_tax_' . $tax);
                    }
                } else {
                    $cmb->add_field( array(
                        'name' => $label,
                        'id' => 'custom_post_tax_' . $tax,
                        'type' => 'text',
                        'attributes' => array(
                            'autocomplete' => 'off',
                            'class' => 'tmuserb-tags'
                        )
                    ) );
                    if ( isset( $entry['required'] ) && $entry['required'] == 'enable' ) {
                        array_push($required_fields, 'custom_post_tax_' . $tax);
                    }
                }
            }
        }

        // Store required field ids

        if (is_array($req_main)) {
            $main_fields = array(
                'post_format' => 'submitted_post_format',
                'content' => 'submitted_post_content',
                'excerpt' => 'submitted_post_excerpt',
                'featured' => 'submitted_post_thumbnail',
            );

            foreach ( $default_fields as $key => $entry ) { 
                if (in_array($entry, $req_main, false)) {
                    array_push($required_fields, $main_fields[$entry]);
                }
            }
        }

        $get_required_fields = implode(",", $required_fields);

        $cmb->add_field( array(
            'id'      => 'required_fields',
            'type'    => 'hidden',
            'save_field' => false,
            'default' => $get_required_fields,
            'attributes' => array(
                'autocomplete' => 'off'
            )
        ));

        $recaptcha_theme = get_post_meta( get_the_id(), 'tmuserb_cmb2_recaptcha_theme', true );

        if (empty($recaptcha_theme)) {
            $recaptcha_theme = 'light';
        }

        $cmb->add_field( array(
            'name'    => esc_html__( 'Save as draft?', 'tmuserblog' ),
            'id'      => 'submitted_draft',
            'show_option_none' => false,
            'type' => 'radio',
            'after_row' => array($this, 'recaptcha'),
            'options'          => array(
                'yes' => esc_html__( 'Yes', 'tmuserblog' ),
                'no' => esc_html__( 'No', 'tmuserblog' ),
            ),
            'attributes' => array(
                'data-theme' => $recaptcha_theme
            ),
            'default' => 'no'
        ));

        endwhile;

        wp_reset_postdata();

    }

    /**
     * Gets the front-end-post-form cmb instance
     *
     * @return CMB2 object
     */
    public function get_form($form_id) {
        // Use ID of metabox in form_register
        $metabox_id = 'tmuseb-front-end-post-form-' . $form_id;

        // Post/object ID is not applicable since we're using this form for submission
        $object_id  = 'fake-object-id';

        // Get CMB2 metabox object
        return cmb2_get_metabox( $metabox_id, $object_id );
    }

    /**
     * Handle the blog shortcode
     *
     * @param  array  $atts Array of shortcode attributes
     * @return string       Form html
     */
    public function blog_shortcode( $atts = array() ) {
        // Load the scripts
        self::$add_scripts = true;

        // Parse attributes
        $atts = extract(shortcode_atts( array(
            'user_id' => 1,
            'post_type' => 'post'
        ), $atts, 'tmuserb_blog' ));

        return $this->user_blog($user_id, $post_type);
    }

    /**
     * Handle the shortcode
     *
     * @param  array  $atts Array of shortcode attributes
     * @return string       Form html
     */
    public function shortcode( $atts = array() ) {
        // Load the scripts
        self::$add_scripts = true;

        $post_status = tmuserb_get_option('tmuserb_options', 'post_status', 'pending');

        // Check if the user is allowed to use the form
        if( is_user_logged_in() && current_user_can('publish_posts')) { 

            $user_id = get_current_user_id();

            // Parse attributes
            $atts = extract(shortcode_atts( array(
                'form_id' => 1,
                'post_author' => $user_id ? $user_id : 1, // Current user, or admin
                'post_type'   => 'post',
            ), $atts, 'tmuserb_form' ));

            // Get CMB2 metabox object
            if ( isset( $_GET['post_edit'] )) {
                $cmb = $this->get_edit_form($form_id, absint($_GET['post_edit']));
            } else {
                $cmb = $this->get_form($form_id);
            }

            // Get $cmb object_types
            $post_type = $cmb->prop( 'object_types' );
            $post_type = reset($post_type);

            /*
            * Let's add these attributes as hidden fields to our cmb form
            * so that they will be passed through to our form submission
            */
            $cmb->add_hidden_field( array(
                'field_args'  => array(
                    'id'    => 'submitted_form_id',
                    'type'  => 'hidden',
                    'default' => $form_id
                ),
            ));
            $cmb->add_hidden_field( array(
                'field_args'  => array(
                    'id'    => 'submitted_post_author',
                    'type'  => 'hidden',
                    'default' => $post_author
                ),
            ));
            $cmb->add_hidden_field( array(
                'field_args'  => array(
                    'id'    => 'submitted_post_type',
                    'type'  => 'hidden',
                    'default' => $post_type
                ),
            ));

            // Initiate our output variable
            $output = '';

            // Get any submission errors
            if ( ( $error = $cmb->prop( 'submission_error' ) ) && is_wp_error( $error ) ) {
                // If there was an error with the submission, add it to our ouput.
                $output .= '<div class="tmuserb-notice error"><p>' . sprintf( esc_html__( 'There was an error in the submission: %s', 'tmuserblog' ), '<strong>'. $error->get_error_message() .'</strong>' ) . '</p><a class="remove-notice" href="#"><i class="tmuserb-icon-delete"></i></a></div>';
            }

            if (strpos($_SERVER['REQUEST_URI'], "trashed=1") !== false && !isset( $_GET['post_submitted'] ) && !isset( $_GET['post_updated'] )){
                echo '<div class="tmuserb-notice success"><p>' . esc_html__( 'The post is deleted.', 'tmuserblog' ) . '</p><a class="remove-notice" href="#"><i class="tmuserb-icon-delete"></i></a></div>';
            }

            // If the post was submitted successfully, notify the user.
            if ( isset( $_GET['post_submitted'] ) && ( $post = get_post( absint( $_GET['post_submitted'] ) ) ) ) {
                // Add notice of submission to our output
                if (get_post_status(absint( $_GET['post_submitted'] )) == 'draft') {
                    $output .= '<div class="tmuserb-notice warning"><p>' . esc_html__( 'Thank you, your new post saved as draft.', 'tmuserblog' ) . '</p><a class="remove-notice" href="#"><i class="tmuserb-icon-delete"></i></a></div>';
                } else {
                    if ($post_status == 'publish' || current_user_can('editor')) {
                        $output .= '<div class="tmuserb-notice success"><p>' . esc_html__( 'Thank you, your new post has been published.', 'tmuserblog' ) . '</p><a class="remove-notice" href="#"><i class="tmuserb-icon-delete"></i></a></div>';
                    } else {
                        $output .= '<div class="tmuserb-notice info"><p>' . esc_html__( 'Thank you, your new post has been submitted and is pending review by a site administrator.', 'tmuserblog' ) . '</p><a class="remove-notice" href="#"><i class="tmuserb-icon-delete"></i></a></div>';
                    }
                }
            }

            // If the post was updated successfully, notify the user.
            if ( isset( $_GET['post_updated'] ) && ( $post = get_post( absint( $_GET['post_updated'] ) ) ) ) {
                // Add notice of update to our output
                if (get_post_status(absint( $_GET['post_updated'] )) == 'draft') {
                    $output .= '<div class="tmuserb-notice warning"><p>' . esc_html__( 'Thank you, the post has been saved as draft.', 'tmuserblog' ) . '</p><a class="remove-notice" href="#"><i class="tmuserb-icon-delete"></i></a></div>';
                } else {
                    if ($post_status == 'publish' || current_user_can('editor')) {
                        $output .= '<div class="tmuserb-notice success"><p>' . esc_html__( 'Thank you, the post has been updated.', 'tmuserblog' ) . '</p><a class="remove-notice" href="#"><i class="tmuserb-icon-delete"></i></a></div>';
                    } else {
                        $output .= '<div class="tmuserb-notice info"><p>' . esc_html__( 'Thank you, the post update has been submitted and is pending review by a site administrator.', 'tmuserblog' ) . '</p><a class="remove-notice" href="#"><i class="tmuserb-icon-delete"></i></a></div>';
                    }
                }
            }

            // Titles
            $html_tag = tmuserb_get_option('tmuserb_options_1','html_tag', 'h3');
            $edit_post_title = tmuserb_get_option('tmuserb_options_1','edit_post_title', esc_html__( 'Edit Post', 'tmuserblog' ));
            $add_new_post_title = tmuserb_get_option('tmuserb_options_1','add_new_post_title', esc_html__( 'Add New Post', 'tmuserblog' ));
            $raw_url = esc_url_raw(remove_query_arg(array_keys($_GET)));
            if (function_exists('bbp_is_single_user_profile') && bbp_is_single_user_profile()) {
                $raw_url = add_query_arg( 'blog', 1, $raw_url);
            }

            if ( isset( $_GET['post_edit'] )) {
                $output .= '<' . esc_html($html_tag) . ' class="tmuserb-edit-form-title"><span>' . esc_html($edit_post_title) . '</span><a class="tmuserb-add-post" href="' . $raw_url . '">' . esc_html($add_new_post_title) . '<i class="tmuserb-icon-long-arrow"></i></a></' . esc_html($html_tag) . '>';
                $output .= cmb2_get_metabox_form( $cmb, absint($_GET['post_edit']), array( 'save_button' => esc_html__( 'Update Post', 'tmuserblog' ) ) );
            } else {
                if (!empty($add_new_post_title)) {
                    $output .= '<' . esc_html($html_tag) . ' class="tmuserb-edit-form-title">' . esc_html($add_new_post_title) . '</' . esc_html($html_tag) . '>';
                }
                $output .= cmb2_get_metabox_form( $cmb, 'fake-object-id', array( 'save_button' => esc_html__( 'Submit Post', 'tmuserblog' ) ) );
            }

            // Navigation
            $first_tab_title = tmuserb_get_option('tmuserb_options_1','first_tab_title', esc_html__( 'Post Form', 'tmuserblog' ));
            $second_tab_title = tmuserb_get_option('tmuserb_options_1','second_tab_title', esc_html__( 'Your Posts', 'tmuserblog' ));
            $tab_nav = '<ul id="tmuserb-tabs"><li><a id="tmuserb-tab-1" href="#" class="tmuserb-tab active" data-toggle-target=".tmuserb-tab-1">' . esc_html($first_tab_title) . '</a></li><li><a id="tmuserb-tab-2" href="#" class="tmuserb-tab" data-toggle-target=".tmuserb-tab-2">' . esc_html($second_tab_title) . '</a></li></ul>';

            $user = wp_get_current_user();
            $display_name = $user->display_name;
            $logout = '<div class="tmuserb-logout">' . esc_html__( 'You are logged in as', 'tmuserblog' ) . ' ' . $display_name . '. <a href="' . wp_logout_url( home_url() ) . '"> ' . esc_html__( '(Logout)', 'tmuserblog' ) . '</a></div>';

            // Content
            $output = '<div id="tmuserb-content" style="visibility:hidden;">' . $tab_nav . '<div class="tmuserb-tab-content tmuserb-tab-1 active">' . $output . $logout . '</div><div class="tmuserb-tab-content tmuserb-tab-2">' . $this->user_posts($user_id, $post_type) . $logout . '</div></div>';

            return $output;

        } else if (is_user_logged_in() && !current_user_can('publish_posts')) {
            return '<div class="tmuserb-notice warning"><p>' . esc_html__( 'You are not allowed to submit post.', 'tmuserblog' ) . '</p></div>';
        } else {
            // Parse attributes to get recptcha theme
            $atts = extract(shortcode_atts( array(
                'form_id' => 1
            ), $atts, 'tmuserb_form' ));

            $recaptcha_theme = get_post_meta( $form_id, 'tmuserb_cmb2_recaptcha_theme', true );
            if (empty($recaptcha_theme)) {
                $recaptcha_theme = 'light';
            }

            // Login Form
            $login_form_args = array(
                'echo'           => false,
                'redirect'       => ( is_ssl() ? 'https://' : 'http://' ) . $_SERVER['HTTP_HOST'] . $_SERVER['REQUEST_URI'],
                'form_id'        => 'tmuserb_login_form',
                'label_username' => esc_html__( 'Username or Email Address', 'tmuserblog' ),
                'label_password' => esc_html__( 'Password', 'tmuserblog' ),
                'label_remember' => esc_html__( 'Remember Me', 'tmuserblog' ),
                'label_log_in'   => esc_html__( 'Log In', 'tmuserblog' ),
                'remember'       => false,
                'value_username' => '',
                'value_remember' => false
            );
            $output = '<div id="tmuserb_login_form_container" class="tmuserb-login-form-container" data-theme="' . $recaptcha_theme . '"><div class="tmuserb-notice info"><p>' . esc_html__( 'Please log in to submit post.', 'tmuserblog' ) . '</p></div>';
            $output .= wp_login_form($login_form_args);
            $output .= '</div>';
            
            return $output;
        }
    }

    /**
     * Get User Blog
     *
     * @return string   Posts html
     */
    public function user_blog($user_id, $post_type) {
        $max_post = tmuserb_get_option('tmuserb_options_1','max_post', 999);
        $wp_query = new WP_Query(
            array('post_type' => $post_type, 'posts_per_page' => $max_post, 'post_status' => 'publish', 'author' => $user_id) 
        );
        $post_count = $wp_query->post_count;

        $output = '<div class="tmuserb-table-container tmuserb-table-container-2" style="visibility:hidden;"><table class="tmuserb-table-2 tmuserb-table cell-border" style="width:100%;">';
        $output .= '<thead><tr><th class="tmuserb-table-title">' . esc_html__( 'Title', 'tmuserblog' ) . '</th><th>' . esc_html__( 'Date', 'tmuserblog' ) . '</th></tr></thead><tbody>';

        while($wp_query->have_posts()) : $wp_query->the_post();

        $output .= '<tr>';
        $output .= '<td class="tmuserb-title"><a href="' . esc_url(get_the_permalink()) . '" target="_blank">' . esc_html(get_the_title()) . '</a></td><td data-sort="' . esc_attr(get_post_timestamp()) . '">' . human_time_diff(get_the_time('U')) . ' ' . esc_html__('ago', 'tmuserblog') . '</td>';
        $output .= '</tr>';

        endwhile;
        wp_reset_postdata();

        $output .= '</tbody></table></div>';

        return $output;
    }

    /**
     * Get User Posts
     *
     * @return string   Posts html
     */
    public function user_posts($user_id, $post_type) {
        $max_post = tmuserb_get_option('tmuserb_options_1','max_post', 999);
        $status = array(
            'publish' => esc_html__( 'Published', 'tmuserblog' ),
            'private' => esc_html__( 'Private', 'tmuserblog' ),
            'pending' => esc_html__( 'Pending Review', 'tmuserblog' ),
            'draft' => esc_html__( 'Draft', 'tmuserblog' )
        );
        $wp_query = new WP_Query(
            array('post_type' => $post_type, 'posts_per_page' => $max_post, 'post_status' => 'any', 'author' => $user_id) 
        );
        $post_count = $wp_query->post_count;

        $output = '<div id="tmuserb-table-container" class="tmuserb-table-container" data-postcount="' . esc_attr($post_count) . '">';
        $output .= '<table id="tmuserb-table" class="tmuserb-table cell-border" style="width:100%;"><thead><tr>';
        $output .= '<th class="tmuserb-table-title">' . esc_html__( 'Title', 'tmuserblog' ) . '</th>';
        $output .= '<th>' . esc_html__( 'Date', 'tmuserblog' ) . '</th>';
        $output .= '<th>' . esc_html__( 'Status', 'tmuserblog' ) . '</th>';
        $output .= '<th class="tmuserb-table-link"></th><th class="tmuserb-table-link"></th></tr></thead><tbody>';

        while($wp_query->have_posts()) : $wp_query->the_post();

        $output .= '<tr>';
        $output .= (get_post_status() == 'publish') ? '<td class="tmuserb-title"><a href="' . esc_url(get_the_permalink()) . '" target="_blank">' . esc_html(get_the_title()) . '</a></td>' : '<td class="tmuserb-title">' . esc_html(get_the_title()) . '</td>';
        $output .= '<td data-sort="' . esc_attr(get_post_timestamp()) . '">' . get_the_time(get_option('date_format')) . '</td>';
        $output .= '<td class="post_' . esc_attr(get_post_status()) . '">' . esc_html($status[get_post_status()]) . '</td>';

        $raw_url = esc_url_raw(remove_query_arg(array_keys($_GET))); 
        if (function_exists('bbp_is_single_user_profile') && bbp_is_single_user_profile()) {
            $raw_url = add_query_arg( 'blog', 1, $raw_url);
        }

        $output .= '<td class="tmuserb-icon-link"><a href="' . esc_url( add_query_arg( 'post_edit', get_the_ID(), $raw_url) ) . '" title="' . esc_attr__( 'Edit', 'tmuserblog' ) . '"><i class="tmuserb-icon tmuserb-icon-edit"></i></a></td>';
        $output .= '<td class="tmuserb-icon-link"><a href="' . get_delete_post_link(get_the_ID()) . '" onclick="return confirm(\'' . esc_attr__( 'Are you sure delete the post?', 'tmuserblog' ) . '\');" title="' . esc_attr__( 'Delete', 'tmuserblog' ) . '"><i class="tmuserb-icon tmuserb-icon-delete"></i></a></td>';
        $output .= '</tr>';

        endwhile;
        wp_reset_postdata();

        $output .= '</tbody></table></div>';
        return $output;

    }

    /**
     * Get Form Class
     *
     * @return string  
     */
    public function get_form_class() {
        return tmuserb_get_option('tmuserb_options_2','form_class', 'form-control');
    }

    /**
     * Get Recaptcha Site Key
     *
     * @return string  
     */
    public function get_site_key() {
        $site_key = tmuserb_get_option('tmuserb_options','site_key', '');
        return $site_key;
    }

    /**
     * Get Recaptcha Secret Key
     *
     * @return string  
     */
    public function get_secret_key() {
        $secret_key = tmuserb_get_option('tmuserb_options','secret_key', '');
        return $secret_key;
    }

    /**
     * Get Recaptcha Output
     *
     * @return string html
     */
    public function recaptcha($field_args, $field ) {
        if (!empty($this->get_site_key())) {
            echo '<div class="g-recaptcha" data-sitekey="' . $this->get_site_key() . '" data-theme="' . $field_args['attributes']['data-theme'] . '"></div>';
        }
    }

    /**
     * Get Supported Post Formats
     *
     * @return array  
     */
    public function get_post_formats() {
        if ( current_theme_supports( 'post-formats' ) ) {
            $output_formats = array();
            $post_formats = get_theme_support( 'post-formats' );
            $format_names = array(
                'aside' => esc_html__( 'Aside', 'tmuserblog' ),
                'gallery' => esc_html__( 'Gallery', 'tmuserblog' ),
                'link' => esc_html__( 'Link', 'tmuserblog' ),
                'image' => esc_html__( 'Image', 'tmuserblog' ),
                'quote' => esc_html__( 'Quote', 'tmuserblog' ),
                'status' => esc_html__( 'Status', 'tmuserblog' ),
                'video' => esc_html__( 'Video', 'tmuserblog' ),
                'audio' => esc_html__( 'Audio', 'tmuserblog' ),
                'chat' => esc_html__( 'chat', 'tmuserblog' ),
            );
            if (is_array($post_formats[0])) {
                $output_formats['standard'] = esc_html__( 'Default', 'tmuserblog' );
                foreach($post_formats[0] as $format) {
                    $output_formats[$format] = $format_names[$format];
                }
            }
            return $output_formats;
        }
    }

    /**
     * Get Field Options
     *
     * @return array  
     */
    public function get_field_options($field_options) {
        if (!empty($field_options)) {
            $finalArray = array();
            $asArr = explode( ',', $field_options );
            foreach( $asArr as $val ){
                $tmp = explode( '=', $val );
                $key = preg_replace('/\s+/', '', $tmp[0]);
                $finalArray[$key] = $tmp[1];
            }
            return $finalArray;
        }
    }

    /**
     * Handles form submission on save. Redirects if save is successful, otherwise sets an error message as a cmb property
     *
     * @return void
     */
    public function handle_submission() {

        // If no form submission, bail
        if ( empty( $_POST ) || !isset( $_POST['submit-cmb'], $_POST['object_id'], $_POST['submitted_form_id'] ) ) {
            return false;
        }

        $post_data = array();
        $required_fields_check = true;

        // Get our shortcode attributes and set them as our initial post_data args
        if ( isset( $_POST['atts'] ) ) {
            foreach ( (array) $_POST['atts'] as $key => $value ) {
                $post_data[ $key ] = sanitize_text_field( $value );
            }
            unset( $_POST['atts'] );
        }

        // Get CMB2 metabox object
        if ( isset( $_GET['post_edit'] )) {
            $cmb = $this->get_edit_form($_POST['submitted_form_id'], absint($_GET['post_edit']));
        } else {
            $cmb = $this->get_form($_POST['submitted_form_id']);
        }

        // Check security nonce
        if ( ! isset( $_POST[ $cmb->nonce() ] ) || ! wp_verify_nonce( $_POST[ $cmb->nonce() ], $cmb->nonce() ) ) {
            return $cmb->prop( 'submission_error', new WP_Error( 'security_fail', esc_html__( 'Security check failed.', 'tmuserblog' ) ) );
        }

        // Google Recaptcha
        if (!empty($this->get_site_key()) && !empty($this->get_secret_key())) {
            if(isset($_POST['g-recaptcha-response'])){
                $captcha = $_POST['g-recaptcha-response'];
            }
            if(!$captcha){
                return $cmb->prop( 'submission_error', new WP_Error( 'security_fail', esc_html__( 'Please check the Recaptcha form.', 'tmuserblog' ) ) );
            }
            $ip = $_SERVER['REMOTE_ADDR'];
            // post request to server
            $url = 'https://www.google.com/recaptcha/api/siteverify?secret=' . urlencode($this->get_secret_key()) .  '&response=' . urlencode($captcha);
            $response = file_get_contents($url);
            $responseKeys = json_decode($response,true);
            // should return JSON with success as true
            if(!$responseKeys["success"]) {
                return $cmb->prop( 'submission_error', new WP_Error( 'security_fail', esc_html__( 'Recaptcha response was incorrect.', 'tmuserblog' ) ) );
            }
        }

        // Check title submitted
        if ( empty( $_POST['submitted_post_title'] ) ) {
            return $cmb->prop( 'submission_error', new WP_Error( 'post_data_missing', esc_html__( 'Post requires a title.', 'tmuserblog' ) ) );
        }

        // Check required fields submitted
        if ( isset($_POST['required_fields']) && !empty( $_POST['required_fields'] ) ) {
            $required_fields = explode( ',', $_POST['required_fields'] );
            $required_final = array();
            foreach( $required_fields as $field ){
                if (isset( $_POST[$field]) && empty( $_POST[$field] )) {
                    array_push($required_final, $field);
                }
            }
            if (!empty($required_final)) {
                $required_output = implode( ',', $required_final );
                return $cmb->prop( 'submission_error', new WP_Error( 'post_data_missing', esc_html__( 'Please fill all the required fields;', 'tmuserblog' ) . '<span id="tmuserb-required-fields" data-req="' . $required_output . '"></span>' ) );
                $required_fields_check = false;
            }
        }

        /**
         * Fetch sanitized values
         */
        $sanitized_values = $cmb->get_sanitized_values( $_POST );

        $post_type = $cmb->prop( 'object_types' );
        $post_data['post_type'] = reset($post_type);

        // Set our post data arguments
        if ( isset( $_POST['submitted_post_title']) && !empty( $_POST['submitted_post_title'] ) ) {
            $post_data['post_title']   = $sanitized_values['submitted_post_title'];
            unset( $sanitized_values['submitted_post_title'] );
        }

        if ( isset( $_POST['submitted_post_content']) && !empty( $_POST['submitted_post_content'] ) ) {
            $post_data['post_content'] = $sanitized_values['submitted_post_content'];
            unset( $sanitized_values['submitted_post_content'] );
        }
        
        if ( isset( $_POST['submitted_post_excerpt']) && !empty( $_POST['submitted_post_excerpt'] ) ) {
            $post_data['post_excerpt'] = $sanitized_values['submitted_post_excerpt'];
            unset( $sanitized_values['submitted_post_excerpt'] );
        }

        if ( isset( $_POST['submitted_draft']) && !empty( $_POST['submitted_draft'] ) && $sanitized_values['submitted_draft'] == 'yes' ) {
            $post_data['post_status'] = 'draft';
        } elseif (current_user_can('editor')) {
            $post_data['post_status'] = 'publish';
        } else {
            $post_data['post_status'] = tmuserb_get_option('tmuserb_options', 'post_status', 'pending');
        }

        if ( isset( $_GET['post_edit'] )) {
            // Update the new post
            $new_submission_id = $_GET['post_edit'];
            $post_data['ID'] = $new_submission_id;
            wp_update_post( $post_data, true );
            // Add custom post meta fields
            foreach($_POST as $key => $value) {
                if (strpos($key, 'post_meta_') === 0) {
                    $key = substr($key, 10);
                    update_post_meta($new_submission_id, $key, $value, false);
                }
            }
        } else {
            // Create the new post
            $new_submission_id = wp_insert_post( $post_data, true );
            // Add custom post meta fields
            foreach($_POST as $key => $value) {
                if (strpos($key, 'post_meta_') === 0) {
                    $key = substr($key, 10);
                    add_post_meta($new_submission_id, $key, $value, false);
                }
            }
        }

        // Add taxonomies
        foreach($_POST as $key => $value) {
            if ((strpos($key, 'post_tax_') === 0) && (!empty($value))) {
                wp_set_post_terms( $new_submission_id, array($value), $key, false );
            }
        }

        // Add custom taxonomies
        foreach($_POST as $key => $value) {
            if ((strpos($key, 'custom_post_tax_') === 0) && (!empty($value))) {
                $tax = substr('custom_post_tax_post_tag', 16);
                wp_set_post_terms( $new_submission_id, array($value), $tax, false );
            }
        }

        // Set post format
        if ( !empty( $_POST['submitted_post_format'] ) ) {
            set_post_format($new_submission_id, $sanitized_values['submitted_post_format']);
            unset( $sanitized_values['submitted_post_format'] );
        }

        // If we hit a snag, update the user
        if ( is_wp_error( $new_submission_id ) ) {
            return $cmb->prop( 'submission_error', $new_submission_id );
        }

        $cmb->save_fields( $new_submission_id, $post_data['post_type'], $sanitized_values );

        /**
         * Other than post_type and post_status, we want
         * our uploaded attachment post to have the same post-data
         */
        unset( $post_data['post_type'] );
        unset( $post_data['post_status'] );

        // Try to upload the featured image
        $img_id = $this->photo_upload($new_submission_id, $post_data);

        // If our photo upload was successful, set the featured image
        if ( $img_id && ! is_wp_error( $img_id ) ) {
            set_post_thumbnail( $new_submission_id, $img_id );
        }

        /*
        * Redirect back to the form page with a query variable with the new post ID.
        * This will help double-submissions with browser refreshes
        */
        if ( isset( $_GET['post_edit'] ) && $required_fields_check) {
            wp_redirect( esc_url_raw( add_query_arg( 'post_updated', $new_submission_id ) ) );
            exit;
        } else {
            wp_redirect( esc_url_raw( add_query_arg( 'post_submitted', $new_submission_id ) ) );
            exit;
        }
    }

    /**
     * Handles uploading a file to a WordPress post
     *
     * @param  int   $post_id              Post ID to upload the photo to
     * @param  array $attachment_post_data Attachement post-data array
     */
    function photo_upload( $post_id, $attachment_post_data = array() ) {
        // Make sure the right files were submitted
        if ( empty( $_FILES ) || ! isset( $_FILES['submitted_post_thumbnail'] ) || isset( $_FILES['submitted_post_thumbnail']['error'] ) && 0 !== $_FILES['submitted_post_thumbnail']['error']
        ) {
            return;
        }
        
        $check = getimagesize($_FILES["submitted_post_thumbnail"]["tmp_name"]);     
        if ($check === false) {
            return;
        }

        // Filter out empty array values
        $files = array_filter( $_FILES['submitted_post_thumbnail'] );

        // Make sure files were submitted at all
        if ( empty( $files ) ) {
            return;
        }
        
        // Make sure to include the WordPress media uploader API if it's not (front-end)
        if ( ! function_exists( 'media_handle_upload' ) ) {
            require_once( ABSPATH . 'wp-admin/includes/image.php' );
            require_once( ABSPATH . 'wp-admin/includes/file.php' );
            require_once( ABSPATH . 'wp-admin/includes/media.php' );
        }

        // Upload the file and send back the attachment post ID
        return media_handle_upload( 'submitted_post_thumbnail', $post_id, $attachment_post_data );
    }

}

/**
 * Returns the main instance of TMuserBlog.
 */
function TMuserBlog() {  
	return TMuserBlog::instance();
}
// Global for backwards compatibility.
$GLOBALS['TMuserBlog'] = TMuserBlog();