<?php
defined( 'ABSPATH' ) || exit;

class TMuserbSettings {
    /* The single instance of the class */
	protected static $_instance = null;

    /* Main Instance */
	public static function instance() {
		if ( is_null( self::$_instance ) ) {
			self::$_instance = new self();
		}
		return self::$_instance;
	}

    /* Constructor */
    public function __construct() {
        add_action( 'cmb2_admin_init', array($this, 'register_metabox') );
        add_action( 'admin_enqueue_scripts',array($this, 'colorpicker_labels'), 99 );
    }

    /**
    * Hook in and register a metabox to handle a plugin options page and adds a menu item.
    */
    public function register_metabox() {
        // GENERAL SETTINGS
        $args = array(
            'id'           => 'tmuserb_options',
            'title'        => esc_html__('TM User Blog', 'tmuserblog'),
            'object_types' => array( 'options-page' ),
            'option_key'   => 'tmuserb_options',
            'tab_group'    => 'tmuserb_options',
            'tab_title'    => esc_html__('General', 'tmuserblog')
        );
        
        // 'tab_group' property is supported in > 2.4.0.
        if ( version_compare( CMB2_VERSION, '2.4.0' ) ) {
            $args['display_cb'] = array($this, 'display_with_tabs');
        }
        
        $main_options = new_cmb2_box( $args );

        $main_options->add_field(
            array(
                'name' => esc_html__( 'Limit Dashboard Access', 'tmuserblog'),  
                'description' => esc_html__( 'Limit dashboard access and remove WordPress admin bar for the following "Restricted Roles".', 'tmuserblog'),  
                'id' => 'limit_dashboard',
                'type' => 'radio_inline',
                'options' => array(
                    'enable' => esc_html__( 'Enable', 'tmuserblog' ),
                    'disable'   => esc_html__( 'Disable', 'tmuserblog' )
                ),
                'default' => 'disable',
            )
        );

        $main_options->add_field(
            array(
                'name' => esc_html__( 'Restricted Roles', 'tmuserblog'), 
                'id' => 'restricted_roles',
                'type' => 'multicheck_inline',
                'options' => $this->get_role_names()
            )
        );
        
        $main_options->add_field(
            array(
                'name' => esc_html__( 'Default Post Status', 'tmuserblog'),  
                'description' => esc_html__( 'The post status is always "published" for site administrators and editors. This setting is for the other user roles.', 'tmuserblog'), 
                'id' => 'post_status',
                'type' => 'radio_inline',
                'options' => array(
                    'pending' => esc_html__( 'Pending', 'tmuserblog' ),
                    'publish'   => esc_html__( 'Published', 'tmuserblog' )
                ),
                'default' => 'pending',
            )
        );

        $main_options->add_field( array(
            'name' => esc_html__( 'bbPress', 'tmuserblog' ),
            'type' => 'title',
            'id'   => 'bbpress_subtitle'
        ) );

        $main_options->add_field(
            array(
                'name' => esc_html__( 'bbPress Support', 'tmuserblog'),  
                'id' => 'bbpress',
                'type' => 'radio_inline',
                'options' => array(
                    'enable' => esc_html__( 'Enable', 'tmuserblog' ),
                    'disable'   => esc_html__( 'Disable', 'tmuserblog' )
                ),
                'default' => 'disable',
            )
        );

        $main_options->add_field(
            array(
                'name' => esc_html__( 'bbPress Form', 'tmuserblog'),  
                'description' => esc_html__( 'Required for bbPress user profile.', 'tmuserblog' ),
                'id' => 'bbpress_form_id',
                'type' => 'select',
                'options' => $this->get_forms()
            )
        );

        $main_options->add_field( array(
            'name' => esc_html__( 'BuddyPress', 'tmuserblog' ),
            'type' => 'title',
            'id'   => 'buddypress_subtitle'
        ) );
        
        $main_options->add_field(
            array(
                'name' => esc_html__( 'BuddyPress Support', 'tmuserblog'),  
                'id' => 'buddypress',
                'type' => 'radio_inline',
                'options' => array(
                    'enable' => esc_html__( 'Enable', 'tmuserblog' ),
                    'disable'   => esc_html__( 'Disable', 'tmuserblog' )
                ),
                'default' => 'disable',
            )
        );

        $main_options->add_field(
            array(
                'name' => esc_html__( 'BuddyPress Form', 'tmuserblog'),  
                'description' => esc_html__( 'Required for BuddyPress user profile.', 'tmuserblog' ),
                'id' => 'buddypress_form_id',
                'type' => 'select',
                'options' => $this->get_forms()
            )
        );

        $main_options->add_field( array(
            'name' => esc_html__( 'Google Recaptcha V2', 'tmuserblog' ),
            'type' => 'title',
            'id'   => 'recaptcha_subtitle'
        ) );

        $main_options->add_field( array(
            'name' => esc_html__( 'Site Key', 'tmuserblog' ),
            'description' => esc_html__( 'You can get the key from https://www.google.com/recaptcha/admin', 'tmuserblog' ),
            'id'   => 'site_key',
            'type' => 'text'
        ) );

        $main_options->add_field( array(
            'name' => esc_html__( 'Secret Key', 'tmuserblog' ),
            'description' => esc_html__( 'You can get the key from https://www.google.com/recaptcha/admin', 'tmuserblog' ),
            'id'   => 'secret_key',
            'type' => 'text'
        ) );
        
        // POST FORM SETTINGS
        $args = array(
            'id'           => 'tmuserb_options_1',
            'title'        => esc_html__('TM User Blog', 'tmuserblog'),
            'menu_title'   => esc_html__('Post Form', 'tmuserblog'),
            'object_types' => array( 'options-page' ),
            'option_key'   => 'tmuserb_options_1',
            'parent_slug'  => 'tmuserb_options',
            'tab_group'    => 'tmuserb_options',
            'tab_title'    => esc_html__('Post Form', 'tmuserblog'),
        );
        
        // 'tab_group' property is supported in > 2.4.0.
        if ( version_compare( CMB2_VERSION, '2.4.0' ) ) {
            $args['display_cb'] = array($this, 'display_with_tabs');
        }
        
        $options_1 = new_cmb2_box( $args );
        
        $options_1->add_field( array(
            'name' => esc_html__( 'Title Length', 'tmuserblog' ),
            'description' => esc_html__( 'Maximum title length.', 'tmuserblog' ),
            'id'   => 'max_title_length',
            'type' => 'text',
            'attributes' => array(
                'type' => 'number',
                'pattern' => '\d*',
            ),
            'default' => 60,
        ) );
        
        $options_1->add_field( array(
            'name' => esc_html__( 'Excerpt Length', 'tmuserblog' ),
            'description' => esc_html__( 'Maximum excerpt length.', 'tmuserblog' ),
            'id'   => 'max_excerpt_length',
            'type' => 'text',
            'attributes' => array(
                'type' => 'number',
                'pattern' => '\d*',
            ),
            'default' => 120,
        ) );

        $options_1->add_field( array(
            'name' => esc_html__( 'Taxonomy', 'tmuserblog' ),
            'description' => esc_html__( 'Maximum number of taxonomy to be displayed.', 'tmuserblog' ),
            'id'   => 'max_tax',
            'type' => 'text',
            'attributes' => array(
                'type' => 'number',
                'pattern' => '\d*',
            ),
            'default' => 60,
        ) );

        $options_1->add_field( array(
            'name' => esc_html__( 'Tabs & Titles', 'tmuserblog' ),
            'type' => 'title',
            'id'   => 'tabs_subtitle'
        ) );
        
        $options_1->add_field( array(
            'name' => esc_html__( 'First Tab Title', 'tmuserblog' ),
            'id'   => 'first_tab_title',
            'type' => 'text',
            'default' => esc_html__( 'Post Form', 'tmuserblog' )
        ) );
        
        $options_1->add_field( array(
            'name' => esc_html__( 'Second Tab Title', 'tmuserblog' ),
            'id'   => 'second_tab_title',
            'type' => 'text',
            'default' => esc_html__( 'Your Posts', 'tmuserblog' )
        ) );
        
        $options_1->add_field(
            array(
                'name' => esc_html__( 'Form Title HTML Tag', 'tmuserblog'),  
                'id' => 'html_tag',
                'type' => 'select',
                'options' => array(
                    'h1' => esc_html__( 'H1', 'tmuserblog' ),
                    'h2' => esc_html__( 'H2', 'tmuserblog' ),
                    'h3' => esc_html__( 'H3', 'tmuserblog' ),
                    'h4' => esc_html__( 'H4', 'tmuserblog' ),
                    'h5' => esc_html__( 'H5', 'tmuserblog' ),
                    'h6' => esc_html__( 'H6', 'tmuserblog' ),
                    'p' => esc_html__( 'P', 'tmuserblog' ),
                    'div' => esc_html__( 'DIV', 'tmuserblog' ),
                ),
                'default' => 'h3',
            )
        );
        
        $options_1->add_field( array(
            'name' => esc_html__( 'Add New Post Form Title', 'tmuserblog' ),
            'id'   => 'add_new_post_title',
            'type' => 'text',
            'default' => esc_html__( 'Add New Post', 'tmuserblog' )
        ) );
        
        $options_1->add_field( array(
            'name' => esc_html__( 'Edit Post Form Title', 'tmuserblog' ),
            'id'   => 'edit_post_title',
            'type' => 'text',
            'default' => esc_html__( 'Edit Post', 'tmuserblog' )
        ) );

        $options_1->add_field( array(
            'name' => esc_html__( 'Rich Text Editor', 'tmuserblog' ),
            'type' => 'title',
            'id'   => 'rich_text_subtitle'
        ) );
        
        $options_1->add_field(
            array(
                'name' => esc_html__( 'Minimal Editor', 'tmuserblog'),  
                'id' => 'teeny',
                'type' => 'radio_inline',
                'options' => array(
                    'enable' => esc_html__( 'Enable', 'tmuserblog' ),
                    'disable'   => esc_html__( 'Disable', 'tmuserblog' )
                ),
                'default' => 'disable',
            )
        );
        
        $options_1->add_field(
            array(
                'name' => esc_html__( 'Editor Quick Tags', 'tmuserblog'),  
                'id' => 'quicktags',
                'type' => 'radio_inline',
                'options' => array(
                    'enable' => esc_html__( 'Enable', 'tmuserblog' ),
                    'disable'   => esc_html__( 'Disable', 'tmuserblog' )
                ),
                'default' => 'disable',
            )
        );
        
        $options_1->add_field(
            array(
                'name' => esc_html__( 'Inline Media Upload', 'tmuserblog'),  
                'description' => esc_html__( 'Subscribers and Contributors can not upload files.', 'tmuserblog'),  
                'id' => 'media_upload',
                'type' => 'radio_inline',
                'options' => array(
                    'enable' => esc_html__( 'Enable', 'tmuserblog' ),
                    'disable'   => esc_html__( 'Disable', 'tmuserblog' )
                ),
                'default' => 'disable',
            )
        );

        $options_1->add_field( array(
            'name' => esc_html__( 'User Posts', 'tmuserblog' ),
            'type' => 'title',
            'id'   => 'user_posts_subtitle'
        ) );

        $options_1->add_field( array(
            'name' => esc_html__( 'Page Length', 'tmuserblog' ),
            'description' => esc_html__( 'Required for numeric pagination in the posts tab.', 'tmuserblog' ),
            'id'   => 'page_length',
            'type' => 'text',
            'attributes' => array(
                'type' => 'number',
                'pattern' => '\d*',
            ),
            'default' => 20,
        ) );
        
        $options_1->add_field( array(
            'name' => esc_html__( 'Max. number of the posts', 'tmuserblog' ),
            'description' => esc_html__( 'Required for the post table in the user posts tab.', 'tmuserblog' ),
            'id'   => 'max_post',
            'type' => 'text',
            'attributes' => array(
                'type' => 'number',
                'pattern' => '\d*',
            ),
            'default' => 999,
        ) );
        
        // STYLE SETTINGS
        $args = array(
            'id'           => 'tmuserb_options_2',
            'title'        => esc_html__('TM User Blog', 'tmuserblog'),
            'menu_title'   => esc_html__('Style', 'tmuserblog'),
            'object_types' => array( 'options-page' ),
            'option_key'   => 'tmuserb_options_2',
            'parent_slug'  => 'tmuserb_options',
            'tab_group'    => 'tmuserb_options',
            'tab_title'    => esc_html__('Style', 'tmuserblog'),
        );
        
        // 'tab_group' property is supported in > 2.4.0.
        if ( version_compare( CMB2_VERSION, '2.4.0' ) ) {
            $args['display_cb'] = array($this, 'display_with_tabs');
        }
        
        $options_2 = new_cmb2_box( $args );

        $options_2->add_field( array(
            'name' => esc_html__( 'Form CSS Class', 'tmuserblog' ),
            'id'   => 'form_class',
            'type' => 'text',
            'default' => 'form-control',
        ) );
        
        $options_2->add_field( array(
            'name' => esc_html__( 'Custom CSS', 'tmuserblog' ),
            'description' => esc_html__( 'The code is added only if there is a "tmuserb_form" shortcode on the page.', 'tmuserblog' ),
            'id' => 'custom_css',
            'type' => 'textarea_code',
            'attributes' => array(
                'data-codeeditor' => json_encode( array(
                    'codemirror' => array(
                        'mode' => 'css'
                    ),
                ) ),
            ),
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Font Sizes', 'tmuserblog' ),
            'type' => 'title',
            'id'   => 'font_sizes_subtitle'
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Default Font Size (px)', 'tmuserblog' ),
            'id'   => 'font_size',
            'type' => 'text',
            'attributes' => array(
                'type' => 'number',
                'pattern' => '\d*',
            ),
            'default' => 15,
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Title Font Size (px)', 'tmuserblog' ),
            'id'   => 'title_font_size',
            'type' => 'text',
            'attributes' => array(
                'type' => 'number',
                'pattern' => '\d*',
            ),
            'default' => 24,
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Small Font Size (px)', 'tmuserblog' ),
            'id'   => 'small_font_size',
            'type' => 'text',
            'attributes' => array(
                'type' => 'number',
                'pattern' => '\d*',
            ),
            'default' => 14,
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Text Colors', 'tmuserblog' ),
            'type' => 'title',
            'id'   => 'font_colors_subtitle'
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Default Color', 'tmuserblog' ),
            'id'   => 'default_color',
            'type' => 'colorpicker',
            'default' => '#666666',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Title & Label Color', 'tmuserblog' ),
            'id'   => 'title_color',
            'type' => 'colorpicker',
            'default' => '#000000',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Link Color', 'tmuserblog' ),
            'id'   => 'link_color',
            'type' => 'colorpicker',
            'default' => '#000000',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Link Color (Hover)', 'tmuserblog' ),
            'id'   => 'link_hover_color',
            'type' => 'colorpicker',
            'default' => '#077CB2',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Form', 'tmuserblog' ),
            'type' => 'title',
            'id'   => 'form_subtitle'
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Background', 'tmuserblog' ),
            'id'   => 'form_bg',
            'type' => 'colorpicker',
            'default' => '#fafafa',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Border Color', 'tmuserblog' ),
            'id'   => 'form_border_color',
            'type' => 'colorpicker',
            'default' => 'rgba(0,0,0,0.1)',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Form Fields', 'tmuserblog' ),
            'type' => 'title',
            'id'   => 'form_fields_subtitle'
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Background', 'tmuserblog' ),
            'id'   => 'form_field_bg',
            'type' => 'colorpicker',
            'default' => '#ffffff',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Text Color', 'tmuserblog' ),
            'id'   => 'form_field_color',
            'type' => 'colorpicker',
            'default' => '#000000',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Border Color', 'tmuserblog' ),
            'id'   => 'form_field_border_color',
            'type' => 'colorpicker',
            'default' => 'rgba(0,0,0,0.15)',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Border Color (Focus)', 'tmuserblog' ),
            'id'   => 'form_field_focus_border_color',
            'type' => 'colorpicker',
            'default' => '#000000',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Button', 'tmuserblog' ),
            'type' => 'title',
            'id'   => 'button_subtitle'
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Background', 'tmuserblog' ),
            'id'   => 'button_bg',
            'type' => 'colorpicker',
            'default' => '#000000',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Text Color', 'tmuserblog' ),
            'id'   => 'button_color',
            'type' => 'colorpicker',
            'default' => '#ffffff',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Background (Hover)', 'tmuserblog' ),
            'id'   => 'button_bg_hover',
            'type' => 'colorpicker',
            'default' => '#077CB2',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Text Color (Hover)', 'tmuserblog' ),
            'id'   => 'button_color_hover',
            'type' => 'colorpicker',
            'default' => '#ffffff',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Table', 'tmuserblog' ),
            'type' => 'title',
            'id'   => 'table_subtitle'
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Background', 'tmuserblog' ),
            'id'   => 'table_bg',
            'type' => 'colorpicker',
            'default' => '#ffffff',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Title Text Color', 'tmuserblog' ),
            'id'   => 'table_title_color',
            'type' => 'colorpicker',
            'default' => '#000000',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Text Color', 'tmuserblog' ),
            'id'   => 'table_color',
            'type' => 'colorpicker',
            'default' => '#666666',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Border Color', 'tmuserblog' ),
            'id'   => 'table_border_color',
            'type' => 'colorpicker',
            'default' => 'rgba(0,0,0,0.1)',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Table Pagination', 'tmuserblog' ),
            'type' => 'title',
            'id'   => 'table_pg_subtitle'
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Background', 'tmuserblog' ),
            'id'   => 'table_pg_bg',
            'type' => 'colorpicker',
            'default' => '#ffffff',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Text Color', 'tmuserblog' ),
            'id'   => 'table_pg_color',
            'type' => 'colorpicker',
            'default' => '#000000',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Background (Hover & Active)', 'tmuserblog' ),
            'id'   => 'table_pg_bg_hover',
            'type' => 'colorpicker',
            'default' => '#000000',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Text Color (Hover & Active)', 'tmuserblog' ),
            'id'   => 'table_pg_color_hover',
            'type' => 'colorpicker',
            'default' => '#ffffff',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Other Colors', 'tmuserblog' ),
            'type' => 'title',
            'id'   => 'others_subtitle'
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Info', 'tmuserblog' ),
            'id'   => 'info_color',
            'type' => 'colorpicker',
            'default' => '#2196F3',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Error', 'tmuserblog' ),
            'id'   => 'error_color',
            'type' => 'colorpicker',
            'default' => '#F44336',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Warning', 'tmuserblog' ),
            'id'   => 'warning_color',
            'type' => 'colorpicker',
            'default' => '#FF9800',
            'options' => array(
                'alpha' => true
            )
        ) );

        $options_2->add_field( array(
            'name' => esc_html__( 'Success', 'tmuserblog' ),
            'id'   => 'success_color',
            'type' => 'colorpicker',
            'default' => '#009688',
            'options' => array(
                'alpha' => true
            )
        ) );
        
    }

    /**
    * Gets navigation tabs array for CMB2 options pages which share the given
    * display_cb param.
    *
    * @param CMB2_Options_Hookup $cmb_options The CMB2_Options_Hookup object.
    *
    * @return array Array of tab information.
    */
    public function page_tabs( $cmb_options ) {
        $tab_group = $cmb_options->cmb->prop( 'tab_group' );
        $tabs      = array();
        
        foreach ( CMB2_Boxes::get_all() as $cmb_id => $cmb ) {
            if ( $tab_group === $cmb->prop( 'tab_group' ) ) {
                $tabs[ $cmb->options_page_keys()[0] ] = $cmb->prop( 'tab_title' )
                    ? $cmb->prop( 'tab_title' )
                    : $cmb->prop( 'title' );
            }
        }
        
        return $tabs;
    }

    /**
    * A CMB2 options-page display callback override which adds tab navigation among
    * CMB2 options pages which share this same display callback.
    *
    * @param CMB2_Options_Hookup $cmb_options The CMB2_Options_Hookup object.
    */
    public function display_with_tabs( $cmb_options ) {
        $tabs = $this->page_tabs( $cmb_options );
        ?>
        <div class="wrap cmb2-options-page option-<?php echo esc_attr($cmb_options->option_key); ?>">
            <?php if ( get_admin_page_title() ) : ?>
                <h2><?php echo wp_kses_post( get_admin_page_title() ); ?></h2>
            <?php endif; ?>
            <h2 class="nav-tab-wrapper">
                <?php foreach ( $tabs as $option_key => $tab_title ) : ?>
                    <a class="nav-tab<?php if ( isset( $_GET['page'] ) && $option_key === $_GET['page'] ) : ?> nav-tab-active<?php endif; ?>" href="<?php menu_page_url( $option_key ); ?>"><?php echo wp_kses_post( $tab_title ); ?></a>
                <?php endforeach; ?>
            </h2>
            <form class="cmb-form" action="<?php echo esc_url( admin_url( 'admin-post.php' ) ); ?>" method="POST" id="<?php echo esc_attr($cmb_options->cmb->cmb_id); ?>" enctype="multipart/form-data" encoding="multipart/form-data">
                <input type="hidden" name="action" value="<?php echo esc_attr( $cmb_options->option_key ); ?>">
                <?php $cmb_options->options_page_metabox(); ?>
                <?php submit_button( esc_attr( $cmb_options->cmb->prop( 'save_button' ) ), 'primary', 'submit-cmb' ); ?>
            </form>
        </div>
    <?php
    }

    /**
    * Get Forms
    */
    public function get_forms() {
        $wp_query = new WP_Query(
            array('post_type' => 'tmuserb', 'posts_per_page' => 99, 'post_status' => 'publish') 
        );
        $array = array();
        while($wp_query->have_posts()) : $wp_query->the_post();
        $array[get_the_id()] = get_the_title();
        endwhile;

        wp_reset_postdata();

        return $array;
    }

    /**
    * Get Role Names Except Administrator & Editor
    */
    public function get_role_names() {
        global $wp_roles; 
        if ( ! isset( $wp_roles ) ) {
            $wp_roles = new WP_Roles(); 
        }
        $roles = $wp_roles->get_names();
        unset($roles['administrator']);
        unset($roles['editor']);
        return $roles;
    }

    /**
    * Colorpicker Labels
    */
    function colorpicker_labels( $hook ) {
        global $wp_version;
        if( version_compare( $wp_version, '5.4.2' , '>=' ) ) {
            wp_localize_script(
            'wp-color-picker',
            'wpColorPickerL10n',
            array(
                'clear'            => esc_html__( 'Clear', 'tmuserblog' ),
                'clearAriaLabel'   => esc_html__( 'Clear color', 'tmuserblog' ),
                'defaultString'    => esc_html__( 'Default', 'tmuserblog' ),
                'defaultAriaLabel' => esc_html__( 'Select default color', 'tmuserblog' ),
                'pick'             => esc_html__( 'Select Color', 'tmuserblog' ),
                'defaultLabel'     => esc_html__( 'Color value', 'tmuserblog' )
            )
            );
        }
    }

}

/**
 * Returns the main instance of TMuserBlog.
 */
function TMuserbSettings() {  
	return TMuserbSettings::instance();
}
// Global for backwards compatibility.
$GLOBALS['TMuserbSettings'] = TMuserbSettings();


/**
* Custom Get option
*/

function tmuserb_get_option( $tab = 'tmuserb_options', $key = '', $default = false ) {
	if ( function_exists( 'cmb2_get_option' ) ) {
		return cmb2_get_option( $tab, $key, $default );
	}
	$opts = get_option( $tab, $default );
	$val = $default;
	if ( 'all' == $key ) {
		$val = $opts;
	} elseif ( is_array( $opts ) && array_key_exists( $key, $opts ) && false !== $opts[ $key ] ) {
		$val = $opts[ $key ];
	}
	return $val;
}