<?php
namespace ElementorTMuserb\Widgets;

use Elementor\Widget_Base;
use Elementor\Controls_Manager;
use Elementor\Group_Control_Background;
use Elementor\Group_Control_Typography;
use Elementor\Group_Control_Border;
use Elementor\Group_Control_Box_Shadow;

if ( ! defined( 'ABSPATH' ) ) exit;

class TMuserb_Form extends Widget_Base {

	public function get_name() {
		return 'tmuserb-form';
	}

	public function get_title() {
		return esc_html__( 'TMuserb Form', 'tmuserblog' );
	}

    public function get_icon() {
		return 'eicon-form-horizontal';
    }

	public function get_categories() {
		return [ 'tmuserb-widgets' ];
	}

    public function get_script_depends() {
		return [ 'tmuserb-elementor-form'];
	}

    public function get_forms() {
        $output = array();
        $args = array(
            'post_type' => 'tmuserb', 
            'posts_per_page' => 99, 
            'post_status' => 'publish'
        );
        $posts = get_posts($args);
        foreach($posts as $post) {
            $output[$post->ID] = $post->post_title;
        }
        return $output;
	}

	protected function register_controls() {

        $this->start_controls_section(
			'table_settings_section',
			[
				'label' => esc_html__( 'Settings', 'tmuserblog' ),
                'tab' => \Elementor\Controls_Manager::TAB_CONTENT,
			]
        );

        $this->add_control(
			'form',
			[
				'label' => esc_html__( 'Select Form', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => $this->get_forms(),
			]
		);

		$this->end_controls_section();
        
        // section start
		$this->start_controls_section(
			'form_style_section',
			[
				'label' => esc_html__( 'Form', 'tmuserblog' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
        );

		$this->add_control(
			'form_layout',
			[
				'label' => esc_html__( 'Form Layout', 'tmusers' ),
				'type' => \Elementor\Controls_Manager::SELECT,
				'options' => array(
					'tmuserb-h-layout' => esc_html__( 'Horizontal', 'tmusers' ),
					'tmuserb-v-layout' => esc_html__( 'Vertical', 'tmusers' )
				),
				'default' => 'tmuserb-h-layout'
			]
		);

		$this->add_control(
			'form_left_width',
			[
				'label' => esc_html__( 'Left Column Width (px)', 'tmuserblog' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 0,
				'max' => 300,
				'step' => 1,
                'default' => 150,
				'condition' => [
                    'form_layout' => 'tmuserb-h-layout'
                ],
                'selectors' => [
					'{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .cmb-row .cmb-th' => 'min-width: {{VALUE}}px;'
				]
			]
		);

		$this->add_control(
			'form_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .tmuserb-tab-content' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} #tmuserb-tabs li a.tmuserb-tab.active' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} #tmuserb-tabs li a.tmuserb-tab:hover' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} .tmuserb-login-form-container' => 'background-color: {{VALUE}};'
				]
			]
        );

		$this->add_control(
			'form_border_color',
			[
				'label' => esc_html__( 'Border Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} .tmuserb-tab-content' => 'border-color: {{VALUE}};',
					'{{WRAPPER}} #tmuserb-tabs li a.tmuserb-tab.active' => 'border-color: {{VALUE}};',
					'{{WRAPPER}} #tmuserb-tabs li a.tmuserb-tab:hover' => 'border-color: {{VALUE}};',
					'{{WRAPPER}} #tmuserb-tabs' => 'border-color: {{VALUE}};',
					'{{WRAPPER}} .tmuserb-login-form-container' => 'border-color: {{VALUE}};',
					'{{WRAPPER}} .cmb2-id-submitted-draft' => 'border-color: {{VALUE}};',
				]
			]
        );

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'label' => esc_html__( 'Typography', 'tmuserblog' ),
				'name' => 'form_typography',
				'selector' => '{{WRAPPER}} .tmuserb-tab-content,{{WRAPPER}} .tmuserb-tab-content p,.tmuserb-notice p',
			]
        );

		$this->add_control(
			'form_color',
			[
				'label' => esc_html__( 'Text Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmuserb-tab-content,{{WRAPPER}} .tmuserb-tab-content p' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'form_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmuserblog' ),
				'selector' => '{{WRAPPER}} #tmuserb-tabs li a.tmuserb-tab.active, {{WRAPPER}} #tmuserb-tabs li a.tmuserb-tab:hover, {{WRAPPER}} .tmuserb-tab-content, .tmuserb-login-form-container'
			]
		);

		$this->add_control(
			'form_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'label' => esc_html__( 'Tabs Typography', 'tmuserblog' ),
				'name' => 'tabs_typography',
				'selector' => '{{WRAPPER}} #tmuserb-tabs li a',
			]
        );

		$this->add_control(
			'tabs_color',
			[
				'label' => esc_html__( 'Tab Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} #tmuserb-tabs li a.tmuserb-tab' => 'color: {{VALUE}};',
					'{{WRAPPER}} #tmuserb-tabs li a.tmuserb-tab.active' => 'color: {{VALUE}};',
					'{{WRAPPER}} #tmuserb-tabs li a.tmuserb-tab:hover' => 'color: {{VALUE}};',
				]
			]
        );

		$this->add_responsive_control(
			'tabs_padding',
			[
				'label' => esc_html__( 'Tab Padding', 'tmuserblog' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} #tmuserb-tabs li a' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
        );

		$this->add_control(
			'form_hr_2',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'label' => esc_html__( 'Title Typography', 'tmuserblog' ),
				'name' => 'form_title_typography',
				'selector' => '{{WRAPPER}} #tmuserb-content .tmuserb-edit-form-title',
			]
        );

		$this->add_control(
			'form_title_color',
			[
				'label' => esc_html__( 'Title Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} #tmuserb-content .tmuserb-edit-form-title,{{WRAPPER}} #tmuserb-content .tmuserb-edit-form-title .tmuserb-add-post' => 'color: {{VALUE}};'
				]
			]
        );

		$this->add_responsive_control(
			'form_title_margin',
			[
				'label' => esc_html__( 'Title Margin', 'tmuserblog' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} #tmuserb-content .tmuserb-edit-form-title' => 'margin: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
        );

		$this->add_control(
			'form_hr_3',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'label' => esc_html__( 'Label Typography', 'tmuserblog' ),
				'name' => 'form_label_typography',
				'selector' => '{{WRAPPER}} div[id^="cmb2-metabox-tmuseb-front-end-post-form"] .cmb-row .cmb-th label,.tmuserb-login-form-container label',
			]
        );

		$this->add_control(
			'form_label_color',
			[
				'label' => esc_html__( 'Label Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} div[id^="cmb2-metabox-tmuseb-front-end-post-form"] .cmb-row .cmb-th label' => 'color: {{VALUE}};',
					'{{WRAPPER}} .tmuserb-login-form-container label' => 'color: {{VALUE}} !important;'
				]
			]
        );

		$this->add_control(
			'form_hr_4',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 

		$this->add_responsive_control(
			'form_field_bottom',
			[
				'label' => esc_html__( 'Row Margin Bottom (px)', 'tmuserblog' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 0,
				'max' => 100,
				'step' => 1,
                'default' => 30,
                'selectors' => [
					'{{WRAPPER}} div[id^="cmb2-metabox-tmuseb-front-end-post-form"] .cmb-row' => 'margin-bottom: {{VALUE}}px;',
					'{{WRAPPER}} div[id^="cmb2-metabox-tmuseb-front-end-post-form"] .cmb-row.cmb2-id-submitted-draft' => 'padding-bottom: {{VALUE}}px;'
				]
			]
		);

		$this->add_responsive_control(
			'form_padding',
			[
				'label' => esc_html__( 'Padding', 'tmuserblog' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .tmuserb-tab-content' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
        );

        $this->end_controls_section();

		// section start
		$this->start_controls_section(
			'form_field_style_section',
			[
				'label' => esc_html__( 'Form Fields', 'tmuserblog' ),
                'tab' => Controls_Manager::TAB_STYLE
			]
        );

        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'form_field_typography',
				'selector' => '{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"],
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"],
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"],
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"],
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] select,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea,
				{{WRAPPER}} #tmuserb_login_form input[type="text"],
				{{WRAPPER}} #tmuserb_login_form input[type="password"]'
			]
		);
        
        $this->add_control(
			'form_field_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 
        
        $this->start_controls_tabs( 'form_field_style' );
        
        $this->start_controls_tab(
			'form_field_normal',
			[
				'label' => esc_html__( 'Normal', 'tmuserblog' ),
			]
		);
        
        $this->add_control(
			'form_field_color',
			[
				'label' => esc_html__( 'Text Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] select,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea,
					{{WRAPPER}} #tmuserb_login_form input[type="text"],
					{{WRAPPER}} #tmuserb_login_form input[type="password"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput div input' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'form_field_placeholder_color',
			[
				'label' => esc_html__( 'Placeholder Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"]::placeholder,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"]::placeholder,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"]::placeholder,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"]::placeholder,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea::placeholder,
					{{WRAPPER}} #tmuserb_login_form input[type="text"]::placeholder,
					{{WRAPPER}} #tmuserb_login_form input[type="password"]::placeholder' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'form_field_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] select,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea,
					{{WRAPPER}} #tmuserb_login_form input[type="text"],
					{{WRAPPER}} #tmuserb_login_form input[type="password"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'form_field_border',
				'label' => esc_html__( 'Border', 'tmuserblog' ),
				'selector' => '{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"],
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"],
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"],
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"],
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] select,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea,
				{{WRAPPER}} #tmuserb_login_form input[type="text"],
				{{WRAPPER}} #tmuserb_login_form input[type="password"],
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput'
			]
		);
        
        $this->add_responsive_control(
			'form_field_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmuserblog' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] select,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea,
					{{WRAPPER}} #tmuserb_login_form input[type="text"],
					{{WRAPPER}} #tmuserb_login_form input[type="password"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'form_field_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmuserblog' ),
				'selector' => '{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"],
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"],
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"],
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"],
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] select,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea,
				{{WRAPPER}} #tmuserb_login_form input[type="text"],
				{{WRAPPER}} #tmuserb_login_form input[type="password"],
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput'
			]
		);
        
        $this->end_controls_tab();

		$this->start_controls_tab(
			'form_field_hover',
			[
				'label' => esc_html__( 'Focus', 'tmuserblog' ),
			]
		);
        
        $this->add_control(
			'form_field_text_hover_color',
			[
				'label' => esc_html__( 'Text Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] select:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea:focus,
					{{WRAPPER}} #tmuserb_login_form input[type="text"]:focus,
					{{WRAPPER}} #tmuserb_login_form input[type="password"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput.focus' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'form_field_bg_hover_color',
			[
				'label' => esc_html__( 'Background Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] select:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea:focus,
					{{WRAPPER}} #tmuserb_login_form input[type="text"]:focus,
					{{WRAPPER}} #tmuserb_login_form input[type="password"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput.focus' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'form_field_hover_border',
				'label' => esc_html__( 'Border', 'tmuserblog' ),
				'selector' => '{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"]:focus,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"]:focus,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"]:focus,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"]:focus,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] select:focus,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea:focus,
				{{WRAPPER}} #tmuserb_login_form input[type="text"]:focus,
				{{WRAPPER}} #tmuserb_login_form input[type="password"]:focus,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput.focus'
			]
		);
        
        $this->add_responsive_control(
			'form_field_border_hover_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmuserblog' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] select:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea:focus,
					{{WRAPPER}} #tmuserb_login_form input[type="text"]:focus,
					{{WRAPPER}} #tmuserb_login_form input[type="password"]:focus,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput.focus' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'form_field_border_hover_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmuserblog' ),
				'selector' => '{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"]:focus,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"]:focus,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"]:focus,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"]:focus,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] select:focus,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea:focus,
				{{WRAPPER}} #tmuserb_login_form input[type="text"]:focus,
				{{WRAPPER}} #tmuserb_login_form input[type="password"]:focus,
				{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput.focus'
			]
		);
        
        $this->end_controls_tab();
        $this->end_controls_tabs();
        
        $this->add_control(
			'form_field_hr_2',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);
        
        $this->add_responsive_control(
			'form_field_padding',
			[
				'label' => esc_html__( 'Padding', 'tmuserblog' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="email"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="number"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="text"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] input[type="url"],
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] select,
					{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] textarea,
					{{WRAPPER}} #tmuserb_login_form input[type="text"],
					{{WRAPPER}} #tmuserb_login_form input[type="password"]' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}} !important;'
				],
			]
        );

		$this->end_controls_section();
        
        // section start
		$this->start_controls_section(
			'btn_style_section',
			[
				'label' => esc_html__( 'Submit Button', 'tmuserblog' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
        );

		$this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'btn_typography',
				'selector' => '{{WRAPPER}} .tmuserb-tab-content .cmb-form input[type="submit"],{{WRAPPER}} #tmuserb_login_form input[type="submit"]'
			]
		);

		$this->add_control(
			'btn_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);

		$this->start_controls_tabs( 'btn_style' );
        
        $this->start_controls_tab(
			'btn_normal',
			[
				'label' => esc_html__( 'Normal', 'theme-masters-elementor'),
			]
		);
        
		$this->add_control(
			'btn_color',
			[
				'label' => esc_html__( 'Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmuserb-tab-content .cmb-form input[type="submit"],{{WRAPPER}} #tmuserb_login_form input[type="submit"],{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput .tag' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'btn_bg',
			[
				'label' => esc_html__( 'Background Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmuserb-tab-content .cmb-form input[type="submit"],{{WRAPPER}} #tmuserb_login_form input[type="submit"],{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput .tag' => 'background: {{VALUE}};'
				]
			]
		);

		$this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'btn_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmuserblog' ),
				'selector' => '{{WRAPPER}} .tmuserb-tab-content .cmb-form input[type="submit"],{{WRAPPER}} #tmuserb_login_form input[type="submit"]'
			]
		);
        
        $this->end_controls_tab();

		$this->start_controls_tab(
			'btn_hover',
			[
				'label' => esc_html__( 'Hover', 'theme-masters-elementor'),
			]
		);

		$this->add_control(
			'btn_hover_color',
			[
				'label' => esc_html__( 'Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmuserb-tab-content .cmb-form input[type="submit"]:hover,
					{{WRAPPER}} .tmuserb-tab-content .cmb-form input[type="submit"]:focus,
					{{WRAPPER}} #tmuserb_login_form input[type="submit"]:hover,
					{{WRAPPER}} #tmuserb_login_form input[type="submit"]:focus' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'btn_bg_hover',
			[
				'label' => esc_html__( 'Background Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .tmuserb-tab-content .cmb-form input[type="submit"]:hover,
					{{WRAPPER}} .tmuserb-tab-content .cmb-form input[type="submit"]:focus,
					{{WRAPPER}} #tmuserb_login_form input[type="submit"]:hover,
					{{WRAPPER}} #tmuserb_login_form input[type="submit"]:focus' => 'background: {{VALUE}};'
				]
			]
		);

		$this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'btn_hover_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmuserblog' ),
				'selector' => '{{WRAPPER}} .tmuserb-tab-content .cmb-form input[type="submit"]:hover,
				{{WRAPPER}} .tmuserb-tab-content .cmb-form input[type="submit"]:focus,
				{{WRAPPER}} #tmuserb_login_form input[type="submit"]:hover,
				{{WRAPPER}} #tmuserb_login_form input[type="submit"]:focus'
			]
		);

        $this->end_controls_tab();
        $this->end_controls_tabs();

		$this->add_control(
			'btn_hr_2',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);
        
        $this->add_responsive_control(
			'btn_padding',
			[
				'label' => esc_html__( 'Padding', 'tmuserblog' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .tmuserb-tab-content .cmb-form input[type="submit"],{{WRAPPER}} #tmuserb_login_form input[type="submit"]' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
        );
        
        $this->end_controls_section();
        
        // section start
		$this->start_controls_section(
			'table_style_section',
			[
				'label' => esc_html__( 'Table', 'tmuserblog' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
        );

        $this->add_responsive_control(
			'table_spacing',
			[
				'label' => esc_html__( 'Inner Spacing (px)', 'tmuserblog' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 0,
				'max' => 100,
				'step' => 1,
                'default' => 10,
                'selectors' => [
					'{{WRAPPER}} table.dataTable thead th' => 'padding: {{VALUE}}px;',
					'{{WRAPPER}} table.dataTable.cell-border tbody td' => 'padding: {{VALUE}}px;'
				]
			]
		);

		$this->add_responsive_control(
			'table_top_bottom_spacing',
			[
				'label' => esc_html__( 'Top-Bottom Spacing (px)', 'tmuserblog' ),
				'type' => \Elementor\Controls_Manager::NUMBER,
				'min' => 0,
				'max' => 100,
				'step' => 1,
                'default' => 15,
                'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_scroll' => 'padding: {{VALUE}}px 0;'
				]
			]
		);
        
        $this->add_control(
			'table_border_color',
			[
				'label' => esc_html__( 'Border Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} table.dataTable.cell-border tbody th' => 'border-color: {{VALUE}} !important;',
                    '{{WRAPPER}} table.dataTable.cell-border tbody td' => 'border-color: {{VALUE}} !important;',
                    '{{WRAPPER}} table.dataTable thead th' => 'border-color: {{VALUE}} !important;',
                    '{{WRAPPER}} table.dataTable thead td' => 'border-color: {{VALUE}} !important;',
                    '{{WRAPPER}} .dataTables_wrapper.no-footer .dataTables_scrollBody' => 'border-color: {{VALUE}} !important;'
				]
			]
        );
        
        $this->end_controls_section();
        
        // section start
		$this->start_controls_section(
			'table_header_style_section',
			[
				'label' => esc_html__( 'Table Header', 'tmuserblog' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
        );

        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'table_header_typography',
				'selector' => '{{WRAPPER}} table.dataTable thead th',
			]
        );

        $this->add_control(
			'table_header_font_color',
			[
				'label' => esc_html__( 'Font Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} table.dataTable thead th' => 'color: {{VALUE}} !important;'
				]
			]
        );

        $this->add_control(
			'table_header_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} table.dataTable thead th' => 'background-color: {{VALUE}} !important;'
				]
			]
		);

        $this->end_controls_section();
        
        // section start
		$this->start_controls_section(
			'table_body_style_section',
			[
				'label' => esc_html__( 'Table Body', 'tmuserblog' ),
				'tab' => Controls_Manager::TAB_STYLE,
			]
        );

        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'table_body_typography',
				'selector' => '{{WRAPPER}} table.dataTable tbody tr',
			]
        );

        $this->add_control(
			'table_body_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 

        $this->start_controls_tabs( 'table_body_styles' );
        
        $this->start_controls_tab(
			'table_body_odd',
			[
				'label' => esc_html__( 'Odd', 'tmuserblog'),
			]
		);
        
        $this->add_control(
			'table_body_font_color_odd',
			[
				'label' => esc_html__( 'Text Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.odd td' => 'color: {{VALUE}} !important;'
				]
			]
        );

        $this->add_control(
			'table_body_link_color_odd',
			[
				'label' => esc_html__( 'Link Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.odd td a' => 'color: {{VALUE}} !important;'
				]
			]
        );

        $this->add_control(
			'table_body_link_hover_color_odd',
			[
				'label' => esc_html__( 'Link Hover Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.odd td a:hover' => 'color: {{VALUE}} !important;'
				]
			]
        );

        $this->add_control(
			'table_body_odd_bg',
			[
				'label' => esc_html__( 'Row Background Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.odd' => 'background-color: {{VALUE}} !important;',
                    '{{WRAPPER}} table.dataTable tbody tr.odd td' => 'background-color: {{VALUE}} !important;'
				]
			]
        );
        
        $this->end_controls_tab();

		$this->start_controls_tab(
			'table_body_even',
			[
				'label' => esc_html__( 'Even', 'tmuserblog'),
			]
		);

        $this->add_control(
			'table_body_font_color_even',
			[
				'label' => esc_html__( 'Text Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.even td' => 'color: {{VALUE}} !important;'
				]
			]
        );

        $this->add_control(
			'table_body_link_color_even',
			[
				'label' => esc_html__( 'Link Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.even td a' => 'color: {{VALUE}} !important;'
				]
			]
        );

        $this->add_control(
			'table_body_link_hover_color_even',
			[
				'label' => esc_html__( 'Link Hover Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.even td a:hover' => 'color: {{VALUE}} !important;'
				]
			]
        );

        $this->add_control(
			'table_body_even_bg',
			[
				'label' => esc_html__( 'Row Background Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
                'selectors' => [
                    '{{WRAPPER}} table.dataTable tbody tr.even' => 'background-color: {{VALUE}} !important;',
                    '{{WRAPPER}} table.dataTable tbody tr.even td' => 'background-color: {{VALUE}} !important;'
				]
			]
        );

        $this->end_controls_tab();
		$this->end_controls_tabs();

        $this->end_controls_section();
        
        // section start
		$this->start_controls_section(
			'table_buttons_style_section',
			[
				'label' => esc_html__( 'Post Status Menu', 'tmuserblog' ),
                'tab' => Controls_Manager::TAB_STYLE
			]
        );

        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'table_btn_typography',
				'selector' => '{{WRAPPER}} button.tmuserb-table-btn',
			]
		);
        
        $this->add_control(
			'table_btn_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);  

        $this->add_control(
			'table_btn_text_color',
			[
				'label' => esc_html__( 'Link Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
                    '{{WRAPPER}} button.tmuserb-table-btn' => 'color: {{VALUE}} !important;'
				]
			]
		);

        $this->add_control(
			'table_btn_text_hover_color',
			[
				'label' => esc_html__( 'Link Hover Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
                    '{{WRAPPER}} button.tmuserb-table-btn:hover' => 'color: {{VALUE}} !important;',
                    '{{WRAPPER}} button.tmuserb-table-btn.active' => 'color: {{VALUE}} !important;'
				]
			]
		);

        $this->end_controls_section();

        // section start
		$this->start_controls_section(
			'table_pagination_style_section',
			[
				'label' => esc_html__( 'Table Pagination', 'tmuserblog' ),
                'tab' => Controls_Manager::TAB_STYLE
			]
        );

        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'table_pagination_typography',
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button',
			]
		);
        
        $this->add_control(
			'table_pagination_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);  
        
        $this->start_controls_tabs( 'tabs_table_pagination_style' );
        
        $this->start_controls_tab(
			'tab_table_pagination_normal',
			[
				'label' => esc_html__( 'Normal', 'tmuserblog' ),
			]
		);
        
        $this->add_control(
			'table_pagination_text_color',
			[
				'label' => esc_html__( 'Text Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'table_pagination_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'table_pagination_border',
				'label' => esc_html__( 'Border', 'tmuserblog' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button'
			]
		);
        
        $this->add_responsive_control(
			'table_pagination_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmuserblog' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'table_pagination_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmuserblog' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button'
			]
		);
        
        $this->end_controls_tab();

		$this->start_controls_tab(
			'tab_table_pagination_hover',
			[
				'label' => esc_html__( 'Hover', 'tmuserblog' ),
			]
		);
        
        $this->add_control(
			'table_pagination_text_hover_color',
			[
				'label' => esc_html__( 'Text Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current' => 'color: {{VALUE}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current:hover' => 'color: {{VALUE}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:hover' => 'color: {{VALUE}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:active' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'table_pagination_bg_hover_color',
			[
				'label' => esc_html__( 'Background Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current:hover' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:hover' => 'background-color: {{VALUE}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:active' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'table_pagination_hover_border',
				'label' => esc_html__( 'Border', 'tmuserblog' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current,{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current:hover,{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:hover,{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:active'
			]
		);
        
        $this->add_responsive_control(
			'table_pagination_border_hover_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmuserblog' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button.current:hover' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:hover' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};',
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_paginate .paginate_button:active' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->end_controls_tab();
        $this->end_controls_tabs();

        $this->end_controls_section();
        
        // section start
		$this->start_controls_section(
			'table_search_style_section',
			[
				'label' => esc_html__( 'Table Search', 'tmuserblog' ),
                'tab' => Controls_Manager::TAB_STYLE
			]
        );

        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'table_search_typography',
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_filter input'
			]
		);
        
        $this->add_control(
			'table_search_hr_1',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		); 
        
        $this->start_controls_tabs( 'table_search_style' );
        
        $this->start_controls_tab(
			'table_search_normal',
			[
				'label' => esc_html__( 'Normal', 'tmuserblog' ),
			]
		);
        
        $this->add_control(
			'table_search_color',
			[
				'label' => esc_html__( 'Text Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_filter input' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'table_search_placeholder_color',
			[
				'label' => esc_html__( 'Placeholder Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_filter input::placeholder' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'table_search_bg_color',
			[
				'label' => esc_html__( 'Background Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_filter input' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'table_search_border',
				'label' => esc_html__( 'Border', 'tmuserblog' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_filter input'
			]
		);
        
        $this->add_responsive_control(
			'table_search_border_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmuserblog' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_filter input' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'table_search_border_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmuserblog' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_filter input'
			]
		);
        
        $this->end_controls_tab();

		$this->start_controls_tab(
			'table_search_hover',
			[
				'label' => esc_html__( 'Focus', 'tmuserblog' ),
			]
		);
        
        $this->add_control(
			'table_search_text_hover_color',
			[
				'label' => esc_html__( 'Text Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_filter input:focus' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->add_control(
			'table_search_bg_hover_color',
			[
				'label' => esc_html__( 'Background Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_filter input:focus' => 'background-color: {{VALUE}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Border::get_type(),
			[
				'name' => 'table_search_hover_border',
				'label' => esc_html__( 'Border', 'tmuserblog' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_filter input:focus'
			]
		);
        
        $this->add_responsive_control(
			'table_search_border_hover_radius',
			[
				'label' => esc_html__( 'Border Radius', 'tmuserblog' ),
				'type' => Controls_Manager::DIMENSIONS,
                'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_filter input:focus' => 'border-top-left-radius: {{TOP}}{{UNIT}};border-top-right-radius: {{RIGHT}}{{UNIT}};border-bottom-right-radius: {{BOTTOM}}{{UNIT}};border-bottom-left-radius: {{LEFT}}{{UNIT}};'
				]
			]
		);
        
        $this->add_group_control(
			Group_Control_Box_Shadow::get_type(),
			[
				'name' => 'table_search_border_hover_shadow',
				'label' => esc_html__( 'Box Shadow', 'tmuserblog' ),
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_filter input:focus'
			]
		);
        
        $this->end_controls_tab();
        $this->end_controls_tabs();
        
        $this->add_control(
			'input_hr_2',
			[
				'type' => \Elementor\Controls_Manager::DIVIDER,
			]
		);
        
        $this->add_responsive_control(
			'table_search_padding',
			[
				'label' => esc_html__( 'Padding', 'tmuserblog' ),
				'type' => \Elementor\Controls_Manager::DIMENSIONS,
				'size_units' => [ 'px', '%', 'rem' ],
				'selectors' => [
                    '{{WRAPPER}} .dataTables_wrapper .dataTables_filter input' => 'padding: {{TOP}}{{UNIT}} {{RIGHT}}{{UNIT}} {{BOTTOM}}{{UNIT}} {{LEFT}}{{UNIT}};'
				],
			]
        );
        
        $this->end_controls_section();
        
        // section start
		$this->start_controls_section(
			'table_info_style_section',
			[
				'label' => esc_html__( 'Table Info', 'tmuserblog' ),
                'tab' => Controls_Manager::TAB_STYLE
			]
        );

        $this->add_group_control(
			Group_Control_Typography::get_type(),
			[
				'name' => 'table_info_typography',
				'selector' => '{{WRAPPER}} .dataTables_wrapper .dataTables_info,{{WRAPPER}} #tmuserb-table_info',
			]
        );

        $this->add_control(
			'table_info_color',
			[
				'label' => esc_html__( 'Text Color', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR, 
				'selectors' => [
					'{{WRAPPER}} .dataTables_wrapper .dataTables_info,{{WRAPPER}} #tmuserb-table_info' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->end_controls_section();

		// section start
		$this->start_controls_section(
			'other_colors_style_section',
			[
				'label' => esc_html__( 'Other Colors', 'tmuserblog' ),
                'tab' => Controls_Manager::TAB_STYLE
			]
        );

		$this->add_control(
			'others_info_color',
			[
				'label' => esc_html__( 'Info', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .tmuserb-notice.info p' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} table.dataTable tbody td.post_private' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'others_error_color',
			[
				'label' => esc_html__( 'Error', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .tmuserb-notice.error p' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} .embed-status .cmb2-remove-wrapper a:hover,{{WRAPPER}} div[id^="cmb2-metabox-tmuseb-front-end-post-form"] .cmb-row .cmb-th label.required-field,table.dataTable tbody td.post_pending,{{WRAPPER}} table.dataTable tbody td .tmuserb-icon-delete,table.dataTable tbody td.dataTables_empty,{{WRAPPER}} div[id^=cmb2-metabox-tmuseb-front-end-post-form] .tagsinput .tag .tag-remove' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'others_warning_color',
			[
				'label' => esc_html__( 'Warning', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .tmuserb-notice.warning p' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} table.dataTable tbody td.post_draft' => 'color: {{VALUE}};'
				]
			]
		);

		$this->add_control(
			'others_success_color',
			[
				'label' => esc_html__( 'Success', 'tmuserblog' ),
				'type' => Controls_Manager::COLOR,
				'selectors' => [
					'{{WRAPPER}} .tmuserb-notice.success p' => 'background-color: {{VALUE}};',
					'{{WRAPPER}} table.dataTable tbody td.post_publish,{{WRAPPER}} table.dataTable tbody td .tmuserb-icon-edit' => 'color: {{VALUE}};'
				]
			]
		);
        
        $this->end_controls_section();
	}
    
    protected function render() {
        $settings = $this->get_settings_for_display();
        if (!empty($settings['form'])) {
            echo '<div class="' . $settings['form_layout'] . '">';
			echo do_shortcode('[tmuserb_form form_id="' . $settings['form'] . '"]');
			echo '</div>';
        } else {
            echo '<div class="tmuserb-notice warning"><p>' . esc_html__( 'Please select a form.', 'tmuserblog' ) . '</p></div>';
        }
	}
}